@@ -1,4 +1,4 @@
-// Copyright (c) 2011 The Chromium Authors. All rights reserved.
+// Copyright 2011 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -8,12 +8,14 @@
 #include <stdint.h>
 
 #include <algorithm>
+#include <string_view>
 
+#include "base/check_op.h"
 #include "base/command_line.h"
+#include "base/containers/fixed_flat_set.h"
 #include "base/files/file_path.h"
 #include "base/i18n/base_i18n_switches.h"
 #include "base/logging.h"
-#include "base/stl_util.h"
 #include "base/strings/string_split.h"
 #include "base/strings/string_util.h"
 #include "base/strings/sys_string_conversions.h"
@@ -24,7 +26,7 @@
 #include "third_party/icu/source/common/unicode/uscript.h"
 #include "third_party/icu/source/i18n/unicode/coll.h"
 
-#if defined(OS_IOS)
+#if BUILDFLAG(IS_IOS)
 #include "base/debug/crash_logging.h"
 #include "base/ios/ios_util.h"
 #endif
@@ -37,17 +39,24 @@ std::string GetLocaleString(const icu::Locale& locale) {
   const char* language = locale.getLanguage();
   const char* country = locale.getCountry();
   const char* variant = locale.getVariant();
+  const char* script = locale.getScript();
 
   std::string result =
       (language != nullptr && *language != '\0') ? language : "und";
 
+  if (script != nullptr && *script != '\0') {
+    result += '-';
+    result += script;
+  }
+
   if (country != nullptr && *country != '\0') {
     result += '-';
     result += country;
   }
 
-  if (variant != nullptr && *variant != '\0')
+  if (variant != nullptr && *variant != '\0') {
     result += '@' + base::ToLowerASCII(variant);
+  }
 
   return result;
 }
@@ -63,10 +72,12 @@ base::i18n::TextDirection GetCharacterDirection(UChar32 character) {
     std::string force_flag =
         command_line->GetSwitchValueASCII(switches::kForceTextDirection);
 
-    if (force_flag == switches::kForceDirectionRTL)
+    if (force_flag == switches::kForceDirectionRTL) {
       return base::i18n::RIGHT_TO_LEFT;
-    if (force_flag == switches::kForceDirectionLTR)
+    }
+    if (force_flag == switches::kForceDirectionLTR) {
       return base::i18n::LEFT_TO_RIGHT;
+    }
   }
   // Now that we have the character, we use ICU in order to query for the
   // appropriate Unicode BiDi character type.
@@ -87,8 +98,7 @@ base::i18n::TextDirection GetCharacterDirection(UChar32 character) {
 
 }  // namespace
 
-namespace base {
-namespace i18n {
+namespace base::i18n {
 
 // Represents the locale-specific ICU text direction.
 static TextDirection g_icu_text_direction = UNKNOWN_DIRECTION;
@@ -106,23 +116,25 @@ std::string GetCanonicalLocale(const std::string& locale) {
 // Convert Chrome locale name to ICU locale name
 std::string ICULocaleName(const std::string& locale_string) {
   // If not Spanish, just return it.
-  if (locale_string.substr(0, 2) != "es")
+  if (locale_string.substr(0, 2) != "es") {
     return locale_string;
+  }
   // Expand es to es-ES.
-  if (LowerCaseEqualsASCII(locale_string, "es"))
+  if (EqualsCaseInsensitiveASCII(locale_string, "es")) {
     return "es-ES";
+  }
   // Map es-419 (Latin American Spanish) to es-FOO depending on the system
   // locale.  If it's es-RR other than es-ES, map to es-RR. Otherwise, map
   // to es-MX (the most populous in Spanish-speaking Latin America).
-  if (LowerCaseEqualsASCII(locale_string, "es-419")) {
+  if (EqualsCaseInsensitiveASCII(locale_string, "es-419")) {
     const icu::Locale& locale = icu::Locale::getDefault();
     std::string language = locale.getLanguage();
     const char* country = locale.getCountry();
-    if (LowerCaseEqualsASCII(language, "es") &&
-      !LowerCaseEqualsASCII(country, "es")) {
-        language += '-';
-        language += country;
-        return language;
+    if (EqualsCaseInsensitiveASCII(language, "es") &&
+        !EqualsCaseInsensitiveASCII(country, "es")) {
+      language += '-';
+      language += country;
+      return language;
     }
     return "es-MX";
   }
@@ -132,7 +144,7 @@ std::string ICULocaleName(const std::string& locale_string) {
 }
 
 void SetICUDefaultLocale(const std::string& locale_string) {
-#if defined(OS_IOS)
+#if BUILDFLAG(IS_IOS)
   static base::debug::CrashKeyString* crash_key_locale =
       base::debug::AllocateCrashKeyString("icu_locale_input",
                                           base::debug::CrashKeySize::Size256);
@@ -170,9 +182,10 @@ bool ICUIsRTL() {
 
 TextDirection GetForcedTextDirection() {
 // On iOS, check for RTL forcing.
-#if defined(OS_IOS)
-  if (base::ios::IsInForcedRTL())
+#if BUILDFLAG(IS_IOS)
+  if (base::ios::IsInForcedRTL()) {
     return base::i18n::RIGHT_TO_LEFT;
+  }
 #endif
 
   base::CommandLine* command_line = base::CommandLine::ForCurrentProcess();
@@ -180,11 +193,13 @@ TextDirection GetForcedTextDirection() {
     std::string force_flag =
         command_line->GetSwitchValueASCII(switches::kForceUIDirection);
 
-    if (force_flag == switches::kForceDirectionLTR)
+    if (force_flag == switches::kForceDirectionLTR) {
       return base::i18n::LEFT_TO_RIGHT;
+    }
 
-    if (force_flag == switches::kForceDirectionRTL)
+    if (force_flag == switches::kForceDirectionRTL) {
       return base::i18n::RIGHT_TO_LEFT;
+    }
   }
 
   return base::i18n::UNKNOWN_DIRECTION;
@@ -193,27 +208,27 @@ TextDirection GetForcedTextDirection() {
 TextDirection GetTextDirectionForLocaleInStartUp(const char* locale_name) {
   // Check for direction forcing.
   TextDirection forced_direction = GetForcedTextDirection();
-  if (forced_direction != UNKNOWN_DIRECTION)
+  if (forced_direction != UNKNOWN_DIRECTION) {
     return forced_direction;
+  }
 
-  // This list needs to be updated in alphabetical order if we add more RTL
-  // locales.
-  static const char kRTLLanguageCodes[][3] = {"ar", "fa", "he", "iw", "ur"};
-  std::vector<StringPiece> locale_split =
+  CHECK(locale_name && locale_name[0]);
+
+  static constexpr auto kRtlLanguageCodes =
+      base::MakeFixedFlatSet<std::string_view>({"ar", "fa", "he", "iw", "ur"});
+  std::vector<std::string_view> locale_split =
       SplitStringPiece(locale_name, "-_", KEEP_WHITESPACE, SPLIT_WANT_ALL);
-  const StringPiece& language_code = locale_split[0];
-  if (std::binary_search(kRTLLanguageCodes,
-                         kRTLLanguageCodes + base::size(kRTLLanguageCodes),
-                         language_code))
-    return RIGHT_TO_LEFT;
-  return LEFT_TO_RIGHT;
+  std::string_view language_code = locale_split[0];
+  return kRtlLanguageCodes.contains(language_code) ? RIGHT_TO_LEFT
+                                                   : LEFT_TO_RIGHT;
 }
 
 TextDirection GetTextDirectionForLocale(const char* locale_name) {
   // Check for direction forcing.
   TextDirection forced_direction = GetForcedTextDirection();
-  if (forced_direction != UNKNOWN_DIRECTION)
+  if (forced_direction != UNKNOWN_DIRECTION) {
     return forced_direction;
+  }
 
   UErrorCode status = U_ZERO_ERROR;
   ULayoutType layout_dir = uloc_getCharacterOrientation(locale_name, &status);
@@ -222,39 +237,38 @@ TextDirection GetTextDirectionForLocale(const char* locale_name) {
   return (layout_dir != ULOC_LAYOUT_RTL) ? LEFT_TO_RIGHT : RIGHT_TO_LEFT;
 }
 
-TextDirection GetFirstStrongCharacterDirection(const string16& text) {
-  const UChar* string = reinterpret_cast<const UChar*>(text.c_str());
+TextDirection GetFirstStrongCharacterDirection(std::u16string_view text) {
   size_t length = text.length();
   size_t position = 0;
   while (position < length) {
     UChar32 character;
     size_t next_position = position;
-    U16_NEXT(string, next_position, length, character);
+    U16_NEXT(text, next_position, length, character);
     TextDirection direction = GetCharacterDirection(character);
-    if (direction != UNKNOWN_DIRECTION)
+    if (direction != UNKNOWN_DIRECTION) {
       return direction;
+    }
     position = next_position;
   }
   return LEFT_TO_RIGHT;
 }
 
-TextDirection GetLastStrongCharacterDirection(const string16& text) {
-  const UChar* string = reinterpret_cast<const UChar*>(text.c_str());
+TextDirection GetLastStrongCharacterDirection(std::u16string_view text) {
   size_t position = text.length();
   while (position > 0) {
     UChar32 character;
     size_t prev_position = position;
-    U16_PREV(string, 0, prev_position, character);
+    U16_PREV(text, 0, prev_position, character);
     TextDirection direction = GetCharacterDirection(character);
-    if (direction != UNKNOWN_DIRECTION)
+    if (direction != UNKNOWN_DIRECTION) {
       return direction;
+    }
     position = prev_position;
   }
   return LEFT_TO_RIGHT;
 }
 
-TextDirection GetStringDirection(const string16& text) {
-  const UChar* string = reinterpret_cast<const UChar*>(text.c_str());
+TextDirection GetStringDirection(std::u16string_view text) {
   size_t length = text.length();
   size_t position = 0;
 
@@ -262,11 +276,12 @@ TextDirection GetStringDirection(const string16& text) {
   while (position < length) {
     UChar32 character;
     size_t next_position = position;
-    U16_NEXT(string, next_position, length, character);
+    U16_NEXT(text, next_position, length, character);
     TextDirection direction = GetCharacterDirection(character);
     if (direction != UNKNOWN_DIRECTION) {
-      if (result != UNKNOWN_DIRECTION && result != direction)
+      if (result != UNKNOWN_DIRECTION && result != direction) {
         return UNKNOWN_DIRECTION;
+      }
       result = direction;
     }
     position = next_position;
@@ -274,37 +289,37 @@ TextDirection GetStringDirection(const string16& text) {
 
   // Handle the case of a string not containing any strong directionality
   // characters defaulting to LEFT_TO_RIGHT.
-  if (result == UNKNOWN_DIRECTION)
-    return LEFT_TO_RIGHT;
-
-  return result;
+  return (result == UNKNOWN_DIRECTION) ? LEFT_TO_RIGHT : result;
 }
 
-#if defined(OS_WIN)
-bool AdjustStringForLocaleDirection(string16* text) {
-  if (!IsRTL() || text->empty())
+#if BUILDFLAG(IS_WIN)
+bool AdjustStringForLocaleDirection(std::u16string* text) {
+  if (!IsRTL() || text->empty()) {
     return false;
+  }
 
   // Marking the string as LTR if the locale is RTL and the string does not
   // contain strong RTL characters. Otherwise, mark the string as RTL.
   bool has_rtl_chars = StringContainsStrongRTLChars(*text);
-  if (!has_rtl_chars)
+  if (!has_rtl_chars) {
     WrapStringWithLTRFormatting(text);
-  else
+  } else {
     WrapStringWithRTLFormatting(text);
+  }
 
   return true;
 }
 
-bool UnadjustStringForLocaleDirection(string16* text) {
-  if (!IsRTL() || text->empty())
+bool UnadjustStringForLocaleDirection(std::u16string* text) {
+  if (!IsRTL() || text->empty()) {
     return false;
+  }
 
   *text = StripWrappingBidiControlCharacters(*text);
   return true;
 }
 #else
-bool AdjustStringForLocaleDirection(string16* text) {
+bool AdjustStringForLocaleDirection(std::u16string* text) {
   // On OS X & GTK the directionality of a label is determined by the first
   // strongly directional character.
   // However, we want to make sure that in an LTR-language-UI all strings are
@@ -330,8 +345,9 @@ bool AdjustStringForLocaleDirection(string16* text) {
   // Unlike Windows, Linux and OS X can correctly display RTL glyphs out of the
   // box so there is no issue with displaying zero-width bidi control characters
   // on any system.  Thus no need for the !IsRTL() check here.
-  if (text->empty())
+  if (text->empty()) {
     return false;
+  }
 
   bool ui_direction_is_rtl = IsRTL();
 
@@ -358,33 +374,32 @@ bool AdjustStringForLocaleDirection(string16* text) {
   return true;
 }
 
-bool UnadjustStringForLocaleDirection(string16* text) {
-  if (text->empty())
+bool UnadjustStringForLocaleDirection(std::u16string* text) {
+  if (text->empty()) {
     return false;
+  }
 
   size_t begin_index = 0;
-  char16 begin = text->at(begin_index);
-  if (begin == kLeftToRightMark ||
-      begin == kRightToLeftMark) {
+  char16_t begin = text->at(begin_index);
+  if (begin == kLeftToRightMark || begin == kRightToLeftMark) {
     ++begin_index;
   }
 
   size_t end_index = text->length() - 1;
-  char16 end = text->at(end_index);
-  if (end == kLeftToRightMark ||
-      end == kRightToLeftMark) {
+  char16_t end = text->at(end_index);
+  if (end == kLeftToRightMark || end == kRightToLeftMark) {
     --end_index;
   }
 
-  string16 unmarked_text =
+  std::u16string unmarked_text =
       text->substr(begin_index, end_index - begin_index + 1);
   *text = StripWrappingBidiControlCharacters(unmarked_text);
   return true;
 }
 
-#endif  // !OS_WIN
+#endif  // !BUILDFLAG(IS_WIN)
 
-void EnsureTerminatedDirectionalFormatting(string16* text) {
+void EnsureTerminatedDirectionalFormatting(std::u16string* text) {
   int count = 0;
   for (auto c : *text) {
     if (c == kLeftToRightEmbeddingMark || c == kRightToLeftEmbeddingMark ||
@@ -394,29 +409,30 @@ void EnsureTerminatedDirectionalFormatting(string16* text) {
       --count;
     }
   }
-  for (int j = 0; j < count; j++)
+  for (int j = 0; j < count; j++) {
     text->push_back(kPopDirectionalFormatting);
+  }
 }
 
-void SanitizeUserSuppliedString(string16* text) {
+void SanitizeUserSuppliedString(std::u16string* text) {
   EnsureTerminatedDirectionalFormatting(text);
   AdjustStringForLocaleDirection(text);
 }
 
-bool StringContainsStrongRTLChars(const string16& text) {
-  const UChar* string = reinterpret_cast<const UChar*>(text.c_str());
+bool StringContainsStrongRTLChars(std::u16string_view text) {
   size_t length = text.length();
   size_t position = 0;
   while (position < length) {
     UChar32 character;
     size_t next_position = position;
-    U16_NEXT(string, next_position, length, character);
+    U16_NEXT(text, next_position, length, character);
 
     // Now that we have the character, we use ICU in order to query for the
     // appropriate Unicode BiDi character type.
     int32_t property = u_getIntPropertyValue(character, UCHAR_BIDI_CLASS);
-    if ((property == U_RIGHT_TO_LEFT) || (property == U_RIGHT_TO_LEFT_ARABIC))
+    if ((property == U_RIGHT_TO_LEFT) || (property == U_RIGHT_TO_LEFT_ARABIC)) {
       return true;
+    }
 
     position = next_position;
   }
@@ -424,9 +440,10 @@ bool StringContainsStrongRTLChars(const string16& text) {
   return false;
 }
 
-void WrapStringWithLTRFormatting(string16* text) {
-  if (text->empty())
+void WrapStringWithLTRFormatting(std::u16string* text) {
+  if (text->empty()) {
     return;
+  }
 
   // Inserting an LRE (Left-To-Right Embedding) mark as the first character.
   text->insert(static_cast<size_t>(0), static_cast<size_t>(1),
@@ -436,9 +453,10 @@ void WrapStringWithLTRFormatting(string16* text) {
   text->push_back(kPopDirectionalFormatting);
 }
 
-void WrapStringWithRTLFormatting(string16* text) {
-  if (text->empty())
+void WrapStringWithRTLFormatting(std::u16string* text) {
+  if (text->empty()) {
     return;
+  }
 
   // Inserting an RLE (Right-To-Left Embedding) mark as the first character.
   text->insert(static_cast<size_t>(0), static_cast<size_t>(1),
@@ -449,49 +467,49 @@ void WrapStringWithRTLFormatting(string16* text) {
 }
 
 void WrapPathWithLTRFormatting(const FilePath& path,
-                               string16* rtl_safe_path) {
+                               std::u16string* rtl_safe_path) {
   // Wrap the overall path with LRE-PDF pair which essentialy marks the
   // string as a Left-To-Right string.
   // Inserting an LRE (Left-To-Right Embedding) mark as the first character.
   rtl_safe_path->push_back(kLeftToRightEmbeddingMark);
-#if defined(OS_MACOSX)
-    rtl_safe_path->append(UTF8ToUTF16(path.value()));
-#elif defined(OS_WIN)
-    rtl_safe_path->append(path.value());
-#else  // defined(OS_POSIX) && !defined(OS_MACOSX)
-    std::wstring wide_path = base::SysNativeMBToWide(path.value());
-    rtl_safe_path->append(WideToUTF16(wide_path));
+#if BUILDFLAG(IS_APPLE)
+  rtl_safe_path->append(UTF8ToUTF16(path.value()));
+#elif BUILDFLAG(IS_WIN)
+  rtl_safe_path->append(AsString16(path.value()));
+#else  // BUILDFLAG(IS_POSIX) && !BUILDFLAG(IS_APPLE)
+  std::wstring wide_path = base::SysNativeMBToWide(path.value());
+  rtl_safe_path->append(WideToUTF16(wide_path));
 #endif
   // Inserting a PDF (Pop Directional Formatting) mark as the last character.
   rtl_safe_path->push_back(kPopDirectionalFormatting);
 }
 
-string16 GetDisplayStringInLTRDirectionality(const string16& text) {
+std::u16string GetDisplayStringInLTRDirectionality(std::u16string_view text) {
   // Always wrap the string in RTL UI (it may be appended to RTL string).
   // Also wrap strings with an RTL first strong character direction in LTR UI.
+  std::u16string text_mutable(text);
   if (IsRTL() || GetFirstStrongCharacterDirection(text) == RIGHT_TO_LEFT) {
-    string16 text_mutable(text);
     WrapStringWithLTRFormatting(&text_mutable);
-    return text_mutable;
   }
-  return text;
+  return text_mutable;
 }
 
-string16 StripWrappingBidiControlCharacters(const string16& text) {
-  if (text.empty())
-    return text;
+std::u16string StripWrappingBidiControlCharacters(std::u16string_view text) {
+  if (text.empty()) {
+    return std::u16string(text);
+  }
   size_t begin_index = 0;
-  char16 begin = text[begin_index];
+  char16_t begin = text[begin_index];
   if (begin == kLeftToRightEmbeddingMark ||
-      begin == kRightToLeftEmbeddingMark ||
-      begin == kLeftToRightOverride ||
-      begin == kRightToLeftOverride)
+      begin == kRightToLeftEmbeddingMark || begin == kLeftToRightOverride ||
+      begin == kRightToLeftOverride) {
     ++begin_index;
+  }
   size_t end_index = text.length() - 1;
-  if (text[end_index] == kPopDirectionalFormatting)
+  if (text[end_index] == kPopDirectionalFormatting) {
     --end_index;
-  return text.substr(begin_index, end_index - begin_index + 1);
+  }
+  return std::u16string(text.substr(begin_index, end_index - begin_index + 1));
 }
 
-}  // namespace i18n
-}  // namespace base
+}  // namespace base::i18n
