@@ -2,6 +2,11 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
+#ifdef UNSAFE_BUFFERS_BUILD
+// TODO(crbug.com/40284755): Remove this and spanify to fix the errors.
+#pragma allow_unsafe_buffers
+#endif
+
 #include "base/rand_util.h"
 
 #include <stddef.h>
@@ -22,62 +27,87 @@ namespace base {
 
 namespace {
 
-const int kIntMin = std::numeric_limits<int>::min();
-const int kIntMax = std::numeric_limits<int>::max();
+constexpr int kIntMin = std::numeric_limits<int>::min();
+constexpr int kIntMax = std::numeric_limits<int>::max();
 
 }  // namespace
 
 TEST(RandUtilTest, RandInt) {
-  EXPECT_EQ(base::RandInt(0, 0), 0);
-  EXPECT_EQ(base::RandInt(kIntMin, kIntMin), kIntMin);
-  EXPECT_EQ(base::RandInt(kIntMax, kIntMax), kIntMax);
+  EXPECT_EQ(RandInt(0, 0), 0);
+  EXPECT_EQ(RandInt(kIntMin, kIntMin), kIntMin);
+  EXPECT_EQ(RandInt(kIntMax, kIntMax), kIntMax);
 
   // Check that the DCHECKS in RandInt() don't fire due to internal overflow.
   // There was a 50% chance of that happening, so calling it 40 times means
   // the chances of this passing by accident are tiny (9e-13).
-  for (int i = 0; i < 40; ++i)
-    base::RandInt(kIntMin, kIntMax);
+  for (int i = 0; i < 40; ++i) {
+    RandInt(kIntMin, kIntMax);
+  }
 }
 
 TEST(RandUtilTest, RandDouble) {
   // Force 64-bit precision, making sure we're not in a 80-bit FPU register.
-  volatile double number = base::RandDouble();
-  EXPECT_GT(1.0, number);
-  EXPECT_LE(0.0, number);
+  volatile double number = RandDouble();
+  EXPECT_LT(number, 1.0);
+  EXPECT_GE(number, 0.0);
 }
 
 TEST(RandUtilTest, RandFloat) {
   // Force 32-bit precision, making sure we're not in an 80-bit FPU register.
-  volatile float number = base::RandFloat();
-  EXPECT_GT(1.f, number);
-  EXPECT_LE(0.f, number);
+  volatile float number = RandFloat();
+  EXPECT_LT(number, 1.0f);
+  EXPECT_GE(number, 0.0f);
+}
+
+TEST(RandUtilTest, RandBool) {
+  // This test should finish extremely quickly unless `RandBool()` can only give
+  // one result value.
+  for (bool seen_false = false, seen_true = false; !seen_false || !seen_true;) {
+    (RandBool() ? seen_true : seen_false) = true;
+  }
 }
 
 TEST(RandUtilTest, RandTimeDelta) {
   {
-    const auto delta =
-        base::RandTimeDelta(-base::Seconds(2), -base::Seconds(1));
-    EXPECT_GE(delta, -base::Seconds(2));
-    EXPECT_LT(delta, -base::Seconds(1));
+    const auto delta = RandTimeDelta(-Seconds(2), -Seconds(1));
+    EXPECT_GE(delta, -Seconds(2));
+    EXPECT_LT(delta, -Seconds(1));
   }
 
   {
-    const auto delta = base::RandTimeDelta(-base::Seconds(2), base::Seconds(2));
-    EXPECT_GE(delta, -base::Seconds(2));
-    EXPECT_LT(delta, base::Seconds(2));
+    const auto delta = RandTimeDelta(-Seconds(2), Seconds(2));
+    EXPECT_GE(delta, -Seconds(2));
+    EXPECT_LT(delta, Seconds(2));
   }
 
   {
-    const auto delta = base::RandTimeDelta(base::Seconds(1), base::Seconds(2));
-    EXPECT_GE(delta, base::Seconds(1));
-    EXPECT_LT(delta, base::Seconds(2));
+    const auto delta = RandTimeDelta(Seconds(1), Seconds(2));
+    EXPECT_GE(delta, Seconds(1));
+    EXPECT_LT(delta, Seconds(2));
   }
 }
 
 TEST(RandUtilTest, RandTimeDeltaUpTo) {
-  const auto delta = base::RandTimeDeltaUpTo(base::Seconds(2));
+  const auto delta = RandTimeDeltaUpTo(Seconds(2));
   EXPECT_FALSE(delta.is_negative());
-  EXPECT_LT(delta, base::Seconds(2));
+  EXPECT_LT(delta, Seconds(2));
+}
+
+TEST(RandUtilTest, RandomizeByPercentage) {
+  EXPECT_EQ(0, RandomizeByPercentage(0, 100));
+  EXPECT_EQ(100, RandomizeByPercentage(100, 0));
+
+  // Check that 10 +/- 200% will eventually produce values in each range
+  // [-10, 0), [0, 10), [10, 20), [20, 30).
+  for (bool a = false, b = false, c = false, d = false; !a || !b || !c || !d;) {
+    const int r = RandomizeByPercentage(10, 200);
+    EXPECT_GE(r, -10);
+    EXPECT_LT(r, 30);
+    a |= (r < 0);
+    b |= (r >= 0 && r < 10);
+    c |= (r >= 10 && r < 20);
+    d |= (r >= 20);
+  }
 }
 
 TEST(RandUtilTest, BitsToOpenEndedUnitInterval) {
@@ -124,25 +154,24 @@ TEST(RandUtilTest, RandBytes) {
   const size_t buffer_size = 50;
   uint8_t buffer[buffer_size];
   memset(buffer, 0, buffer_size);
-  base::RandBytes(buffer);
+  RandBytes(buffer);
   std::sort(buffer, buffer + buffer_size);
   // Probability of occurrence of less than 25 unique bytes in 50 random bytes
   // is below 10^-25.
   EXPECT_GT(std::unique(buffer, buffer + buffer_size) - buffer, 25);
 }
 
-// Verify that calling base::RandBytes with an empty buffer doesn't fail.
+// Verify that calling RandBytes with an empty buffer doesn't fail.
 TEST(RandUtilTest, RandBytes0) {
-  base::RandBytes(span<uint8_t>());
-  base::RandBytes(nullptr, 0);
+  RandBytes(span<uint8_t>());
 }
 
 TEST(RandUtilTest, RandBytesAsVector) {
-  std::vector<uint8_t> random_vec = base::RandBytesAsVector(0);
+  std::vector<uint8_t> random_vec = RandBytesAsVector(0);
   EXPECT_TRUE(random_vec.empty());
-  random_vec = base::RandBytesAsVector(1);
+  random_vec = RandBytesAsVector(1);
   EXPECT_EQ(1U, random_vec.size());
-  random_vec = base::RandBytesAsVector(145);
+  random_vec = RandBytesAsVector(145);
   EXPECT_EQ(145U, random_vec.size());
   char accumulator = 0;
   for (auto i : random_vec) {
@@ -154,13 +183,14 @@ TEST(RandUtilTest, RandBytesAsVector) {
 }
 
 TEST(RandUtilTest, RandBytesAsString) {
-  std::string random_string = base::RandBytesAsString(1);
+  std::string random_string = RandBytesAsString(1);
   EXPECT_EQ(1U, random_string.size());
-  random_string = base::RandBytesAsString(145);
+  random_string = RandBytesAsString(145);
   EXPECT_EQ(145U, random_string.size());
   char accumulator = 0;
-  for (auto i : random_string)
+  for (auto i : random_string) {
     accumulator |= i;
+  }
   // In theory this test can fail, but it won't before the universe dies of
   // heat death.
   EXPECT_NE(0, accumulator);
@@ -169,7 +199,7 @@ TEST(RandUtilTest, RandBytesAsString) {
 // Make sure that it is still appropriate to use RandGenerator in conjunction
 // with std::random_shuffle().
 TEST(RandUtilTest, RandGeneratorForRandomShuffle) {
-  EXPECT_EQ(base::RandGenerator(1), 0U);
+  EXPECT_EQ(RandGenerator(1), 0U);
   EXPECT_LE(std::numeric_limits<ptrdiff_t>::max(),
             std::numeric_limits<int64_t>::max());
 }
@@ -179,7 +209,7 @@ TEST(RandUtilTest, RandGeneratorIsUniform) {
   // regression test that consistently failed when RandGenerator was
   // implemented this way:
   //
-  //   return base::RandUint64() % max;
+  //   return RandUint64() % max;
   //
   // A degenerate case for such an implementation is e.g. a top of
   // range that is 2/3rds of the way to MAX_UINT64, in which case the
@@ -197,7 +227,7 @@ TEST(RandUtilTest, RandGeneratorIsUniform) {
   double cumulative_average = 0.0;
   int count = 0;
   while (count < kMaxAttempts) {
-    uint64_t value = base::RandGenerator(kTopOfRange);
+    uint64_t value = RandGenerator(kTopOfRange);
     cumulative_average = (count * cumulative_average + value) / (count + 1);
 
     // Don't quit too quickly for things to start converging, or we may have
@@ -224,12 +254,13 @@ TEST(RandUtilTest, RandUint64ProducesBothValuesOfAllBits) {
   uint64_t found_zeros = kAllOnes;
 
   for (size_t i = 0; i < 1000; ++i) {
-    uint64_t value = base::RandUint64();
+    uint64_t value = RandUint64();
     found_ones |= value;
     found_zeros &= value;
 
-    if (found_zeros == kAllZeros && found_ones == kAllOnes)
+    if (found_zeros == kAllZeros && found_ones == kAllOnes) {
       return;
+    }
   }
 
   FAIL() << "Didn't achieve all bit values in maximum number of tries.";
@@ -238,11 +269,11 @@ TEST(RandUtilTest, RandUint64ProducesBothValuesOfAllBits) {
 TEST(RandUtilTest, RandBytesLonger) {
   // Fuchsia can only retrieve 256 bytes of entropy at a time, so make sure we
   // handle longer requests than that.
-  std::string random_string0 = base::RandBytesAsString(255);
+  std::string random_string0 = RandBytesAsString(255);
   EXPECT_EQ(255u, random_string0.size());
-  std::string random_string1 = base::RandBytesAsString(1023);
+  std::string random_string1 = RandBytesAsString(1023);
   EXPECT_EQ(1023u, random_string1.size());
-  std::string random_string2 = base::RandBytesAsString(4097);
+  std::string random_string2 = RandBytesAsString(4097);
   EXPECT_EQ(4097u, random_string2.size());
 }
 
@@ -255,11 +286,12 @@ TEST(RandUtilTest, DISABLED_RandBytesPerf) {
   const int kTestIterations = 10;
   const size_t kTestBufferSize = 1 * 1024 * 1024;
 
-  std::unique_ptr<uint8_t[]> buffer(new uint8_t[kTestBufferSize]);
-  const base::TimeTicks now = base::TimeTicks::Now();
-  for (int i = 0; i < kTestIterations; ++i)
-    base::RandBytes(make_span(buffer.get(), kTestBufferSize));
-  const base::TimeTicks end = base::TimeTicks::Now();
+  std::array<uint8_t, kTestBufferSize> buffer;
+  const TimeTicks now = TimeTicks::Now();
+  for (int i = 0; i < kTestIterations; ++i) {
+    RandBytes(buffer);
+  }
+  const TimeTicks end = TimeTicks::Now();
 
   LOG(INFO) << "RandBytes(" << kTestBufferSize
             << ") took: " << (end - now).InMicroseconds() << "µs";
@@ -280,8 +312,9 @@ TEST(RandUtilTest, InsecureRandomGeneratorProducesBothValuesOfAllBits) {
     found_ones |= value;
     found_zeros &= value;
 
-    if (found_zeros == kAllZeros && found_ones == kAllOnes)
+    if (found_zeros == kAllZeros && found_ones == kAllOnes) {
       return;
+    }
   }
 
   FAIL() << "Didn't achieve all bit values in maximum number of tries.";
@@ -384,4 +417,53 @@ TEST(RandUtilTest, InsecureRandomGeneratorRandDouble) {
     EXPECT_LT(x, 1.);
   }
 }
+
+TEST(RandUtilTest, MetricsSubSampler) {
+  MetricsSubSampler sub_sampler;
+  int true_count = 0;
+  int false_count = 0;
+  for (int i = 0; i < 1000; ++i) {
+    if (sub_sampler.ShouldSample(0.5)) {
+      ++true_count;
+    } else {
+      ++false_count;
+    }
+  }
+
+  // Validate that during normal operation MetricsSubSampler::ShouldSample()
+  // does not always give the same result. It's technically possible to fail
+  // this test during normal operation but if the sampling is realistic it
+  // should happen about once every 2^999 times (the likelihood of the [1,999]
+  // results being the same as [0], which can be either). This should not make
+  // this test flaky in the eyes of automated testing.
+  EXPECT_GT(true_count, 0);
+  EXPECT_GT(false_count, 0);
+}
+
+TEST(RandUtilTest, MetricsSubSamplerTestingSupport) {
+  MetricsSubSampler sub_sampler;
+
+  // ScopedAlwaysSampleForTesting makes ShouldSample() return true with
+  // any probability.
+  {
+    MetricsSubSampler::ScopedAlwaysSampleForTesting always_sample;
+    for (int i = 0; i < 100; ++i) {
+      EXPECT_TRUE(sub_sampler.ShouldSample(0));
+      EXPECT_TRUE(sub_sampler.ShouldSample(0.5));
+      EXPECT_TRUE(sub_sampler.ShouldSample(1));
+    }
+  }
+
+  // ScopedNeverSampleForTesting makes ShouldSample() return true with
+  // any probability.
+  {
+    MetricsSubSampler::ScopedNeverSampleForTesting always_sample;
+    for (int i = 0; i < 100; ++i) {
+      EXPECT_FALSE(sub_sampler.ShouldSample(0));
+      EXPECT_FALSE(sub_sampler.ShouldSample(0.5));
+      EXPECT_FALSE(sub_sampler.ShouldSample(1));
+    }
+  }
+}
+
 }  // namespace base
