@@ -1,31 +1,31 @@
-// Copyright 2016 The Chromium Authors. All rights reserved.
+// Copyright 2016 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "base/run_loop.h"
 
 #include <functional>
+#include <memory>
 #include <utility>
 
-#include "base/bind.h"
-#include "base/bind_helpers.h"
 #include "base/containers/queue.h"
+#include "base/functional/bind.h"
+#include "base/functional/callback_helpers.h"
 #include "base/location.h"
-#include "base/macros.h"
 #include "base/memory/ptr_util.h"
 #include "base/memory/ref_counted.h"
-#include "base/single_thread_task_runner.h"
 #include "base/synchronization/lock.h"
 #include "base/synchronization/waitable_event.h"
-#include "base/test/bind_test_util.h"
+#include "base/task/sequenced_task_runner.h"
+#include "base/task/single_thread_task_runner.h"
+#include "base/test/bind.h"
 #include "base/test/gtest_util.h"
-#include "base/test/scoped_task_environment.h"
+#include "base/test/scoped_run_loop_timeout.h"
+#include "base/test/task_environment.h"
 #include "base/test/test_timeouts.h"
 #include "base/threading/platform_thread.h"
-#include "base/threading/sequenced_task_runner_handle.h"
 #include "base/threading/thread.h"
 #include "base/threading/thread_checker_impl.h"
-#include "base/threading/thread_task_runner_handle.h"
 #include "build/build_config.h"
 #include "testing/gmock/include/gmock/gmock.h"
 #include "testing/gtest/include/gtest/gtest.h"
@@ -43,12 +43,12 @@ void RunNestedLoopTask(int* counter) {
   RunLoop nested_run_loop(RunLoop::Type::kNestableTasksAllowed);
 
   // This task should quit |nested_run_loop| but not the main RunLoop.
-  ThreadTaskRunnerHandle::Get()->PostTask(
+  SingleThreadTaskRunner::GetCurrentDefault()->PostTask(
       FROM_HERE, BindOnce(&QuitWhenIdleTask, Unretained(&nested_run_loop),
                           Unretained(counter)));
 
-  ThreadTaskRunnerHandle::Get()->PostDelayedTask(
-      FROM_HERE, MakeExpectedNotRunClosure(FROM_HERE), TimeDelta::FromDays(1));
+  SingleThreadTaskRunner::GetCurrentDefault()->PostDelayedTask(
+      FROM_HERE, MakeExpectedNotRunClosure(FROM_HERE), Days(1));
 
   nested_run_loop.Run();
 
@@ -61,12 +61,16 @@ void RunNestedLoopTask(int* counter) {
 class SimpleSingleThreadTaskRunner : public SingleThreadTaskRunner {
  public:
   SimpleSingleThreadTaskRunner() = default;
+  SimpleSingleThreadTaskRunner(const SimpleSingleThreadTaskRunner&) = delete;
+  SimpleSingleThreadTaskRunner& operator=(const SimpleSingleThreadTaskRunner&) =
+      delete;
 
   bool PostDelayedTask(const Location& from_here,
                        OnceClosure task,
                        base::TimeDelta delay) override {
-    if (delay > base::TimeDelta())
+    if (delay.is_positive()) {
       return false;
+    }
     AutoLock auto_lock(tasks_lock_);
     pending_tasks_.push(std::move(task));
     return true;
@@ -86,8 +90,9 @@ class SimpleSingleThreadTaskRunner : public SingleThreadTaskRunner {
     OnceClosure task;
     {
       AutoLock auto_lock(tasks_lock_);
-      if (pending_tasks_.empty())
+      if (pending_tasks_.empty()) {
         return false;
+      }
       task = std::move(pending_tasks_.front());
       pending_tasks_.pop();
     }
@@ -107,8 +112,6 @@ class SimpleSingleThreadTaskRunner : public SingleThreadTaskRunner {
   // ThreadCheckerImpl to be able to reliably provide that signal even in
   // non-dcheck builds.
   ThreadCheckerImpl origin_thread_checker_;
-
-  DISALLOW_COPY_AND_ASSIGN(SimpleSingleThreadTaskRunner);
 };
 
 // The basis of all TestDelegates, allows safely injecting a OnceClosure to be
@@ -125,8 +128,9 @@ class InjectableTestDelegate : public RunLoop::Delegate {
 
   bool RunInjectedClosure() {
     AutoLock auto_lock(closure_lock_);
-    if (closure_.is_null())
+    if (closure_.is_null()) {
       return false;
+    }
     std::move(closure_).Run();
     return true;
   }
@@ -144,10 +148,11 @@ class TestBoundDelegate final : public InjectableTestDelegate {
   TestBoundDelegate() = default;
 
   // Makes this TestBoundDelegate become the RunLoop::Delegate and
-  // ThreadTaskRunnerHandle for this thread.
+  // SingleThreadTaskRunner::CurrentDefaultHandle for this thread.
   void BindToCurrentThread() {
     thread_task_runner_handle_ =
-        std::make_unique<ThreadTaskRunnerHandle>(simple_task_runner_);
+        std::make_unique<SingleThreadTaskRunner::CurrentDefaultHandle>(
+            simple_task_runner_);
     RunLoop::RegisterDelegateForCurrentThread(this);
   }
 
@@ -162,14 +167,18 @@ class TestBoundDelegate final : public InjectableTestDelegate {
     nested_run_allowing_tasks_incoming_ = false;
 
     while (!should_quit_) {
-      if (application_tasks_allowed && simple_task_runner_->ProcessSingleTask())
+      if (application_tasks_allowed &&
+          simple_task_runner_->ProcessSingleTask()) {
         continue;
+      }
 
-      if (ShouldQuitWhenIdle())
+      if (ShouldQuitWhenIdle()) {
         break;
+      }
 
-      if (RunInjectedClosure())
+      if (RunInjectedClosure()) {
         continue;
+      }
 
       PlatformThread::YieldCurrentThread();
     }
@@ -189,13 +198,14 @@ class TestBoundDelegate final : public InjectableTestDelegate {
   scoped_refptr<SimpleSingleThreadTaskRunner> simple_task_runner_ =
       MakeRefCounted<SimpleSingleThreadTaskRunner>();
 
-  std::unique_ptr<ThreadTaskRunnerHandle> thread_task_runner_handle_;
+  std::unique_ptr<SingleThreadTaskRunner::CurrentDefaultHandle>
+      thread_task_runner_handle_;
 
   bool should_quit_ = false;
 };
 
 enum class RunLoopTestType {
-  // Runs all RunLoopTests under a ScopedTaskEnvironment to make sure real world
+  // Runs all RunLoopTests under a TaskEnvironment to make sure real world
   // scenarios work.
   kRealEnvironment,
 
@@ -208,10 +218,10 @@ enum class RunLoopTestType {
 // so it can be instantiated on the stack in the RunLoopTest fixture.
 class RunLoopTestEnvironment {
  public:
-  RunLoopTestEnvironment(RunLoopTestType type) {
+  explicit RunLoopTestEnvironment(RunLoopTestType type) {
     switch (type) {
       case RunLoopTestType::kRealEnvironment: {
-        task_environment_ = std::make_unique<test::ScopedTaskEnvironment>();
+        task_environment_ = std::make_unique<test::TaskEnvironment>();
         break;
       }
       case RunLoopTestType::kTestDelegate: {
@@ -225,32 +235,33 @@ class RunLoopTestEnvironment {
 
  private:
   // Instantiates one or the other based on the RunLoopTestType.
-  std::unique_ptr<test::ScopedTaskEnvironment> task_environment_;
+  std::unique_ptr<test::TaskEnvironment> task_environment_;
   std::unique_ptr<InjectableTestDelegate> test_delegate_;
 };
 
 class RunLoopTest : public testing::TestWithParam<RunLoopTestType> {
+ public:
+  RunLoopTest(const RunLoopTest&) = delete;
+  RunLoopTest& operator=(const RunLoopTest&) = delete;
+
  protected:
   RunLoopTest() : test_environment_(GetParam()) {}
 
   RunLoopTestEnvironment test_environment_;
   RunLoop run_loop_;
-
- private:
-  DISALLOW_COPY_AND_ASSIGN(RunLoopTest);
 };
 
 }  // namespace
 
 TEST_P(RunLoopTest, QuitWhenIdle) {
   int counter = 0;
-  ThreadTaskRunnerHandle::Get()->PostTask(
+  SingleThreadTaskRunner::GetCurrentDefault()->PostTask(
       FROM_HERE, BindOnce(&QuitWhenIdleTask, Unretained(&run_loop_),
                           Unretained(&counter)));
-  ThreadTaskRunnerHandle::Get()->PostTask(FROM_HERE,
-                                          MakeExpectedRunClosure(FROM_HERE));
-  ThreadTaskRunnerHandle::Get()->PostDelayedTask(
-      FROM_HERE, MakeExpectedNotRunClosure(FROM_HERE), TimeDelta::FromDays(1));
+  SingleThreadTaskRunner::GetCurrentDefault()->PostTask(
+      FROM_HERE, MakeExpectedRunClosure(FROM_HERE));
+  SingleThreadTaskRunner::GetCurrentDefault()->PostDelayedTask(
+      FROM_HERE, MakeExpectedNotRunClosure(FROM_HERE), Days(1));
 
   run_loop_.Run();
   EXPECT_EQ(1, counter);
@@ -258,148 +269,31 @@ TEST_P(RunLoopTest, QuitWhenIdle) {
 
 TEST_P(RunLoopTest, QuitWhenIdleNestedLoop) {
   int counter = 0;
-  ThreadTaskRunnerHandle::Get()->PostTask(
+  SingleThreadTaskRunner::GetCurrentDefault()->PostTask(
       FROM_HERE, BindOnce(&RunNestedLoopTask, Unretained(&counter)));
-  ThreadTaskRunnerHandle::Get()->PostTask(
+  SingleThreadTaskRunner::GetCurrentDefault()->PostTask(
       FROM_HERE, BindOnce(&QuitWhenIdleTask, Unretained(&run_loop_),
                           Unretained(&counter)));
-  ThreadTaskRunnerHandle::Get()->PostTask(FROM_HERE,
-                                          MakeExpectedRunClosure(FROM_HERE));
-  ThreadTaskRunnerHandle::Get()->PostDelayedTask(
-      FROM_HERE, MakeExpectedNotRunClosure(FROM_HERE), TimeDelta::FromDays(1));
+  SingleThreadTaskRunner::GetCurrentDefault()->PostTask(
+      FROM_HERE, MakeExpectedRunClosure(FROM_HERE));
+  SingleThreadTaskRunner::GetCurrentDefault()->PostDelayedTask(
+      FROM_HERE, MakeExpectedNotRunClosure(FROM_HERE), Days(1));
 
   run_loop_.Run();
   EXPECT_EQ(3, counter);
 }
 
 TEST_P(RunLoopTest, QuitWhenIdleClosure) {
-  ThreadTaskRunnerHandle::Get()->PostTask(FROM_HERE,
-                                          run_loop_.QuitWhenIdleClosure());
-  ThreadTaskRunnerHandle::Get()->PostTask(FROM_HERE,
-                                          MakeExpectedRunClosure(FROM_HERE));
-  ThreadTaskRunnerHandle::Get()->PostDelayedTask(
-      FROM_HERE, MakeExpectedNotRunClosure(FROM_HERE), TimeDelta::FromDays(1));
+  SingleThreadTaskRunner::GetCurrentDefault()->PostTask(
+      FROM_HERE, run_loop_.QuitWhenIdleClosure());
+  SingleThreadTaskRunner::GetCurrentDefault()->PostTask(
+      FROM_HERE, MakeExpectedRunClosure(FROM_HERE));
+  SingleThreadTaskRunner::GetCurrentDefault()->PostDelayedTask(
+      FROM_HERE, MakeExpectedNotRunClosure(FROM_HERE), Days(1));
 
   run_loop_.Run();
 }
 
-TEST_P(RunLoopTest, RunWithTimeout) {
-  // SimpleSingleThreadTaskRunner doesn't support delayed tasks.
-  if (GetParam() == RunLoopTestType::kTestDelegate)
-    return;
-
-  bool task1_run = false;
-  bool task2_run = false;
-  bool task3_run = false;
-  ThreadTaskRunnerHandle::Get()->PostDelayedTask(
-      FROM_HERE, BindLambdaForTesting([&]() { task1_run = true; }),
-      TimeDelta::FromMilliseconds(10));
-
-  ThreadTaskRunnerHandle::Get()->PostDelayedTask(
-      FROM_HERE, BindLambdaForTesting([&]() { task2_run = true; }),
-      TimeDelta::FromMilliseconds(20));
-
-  ThreadTaskRunnerHandle::Get()->PostDelayedTask(
-      FROM_HERE, BindLambdaForTesting([&]() { task3_run = true; }),
-      TimeDelta::FromSeconds(10));
-
-  run_loop_.RunWithTimeout(TimeDelta::FromMilliseconds(20));
-  EXPECT_TRUE(task1_run);
-  EXPECT_TRUE(task2_run);
-  EXPECT_FALSE(task3_run);
-}
-
-// TODO(https://crbug.com/970187): This test is inherently flaky.
-TEST_P(RunLoopTest, DISABLED_NestedRunWithTimeout) {
-  // SimpleSingleThreadTaskRunner doesn't support delayed tasks.
-  if (GetParam() == RunLoopTestType::kTestDelegate)
-    return;
-
-  bool task1_run = false;
-  bool task2_run = false;
-  bool task3_run = false;
-  bool task4_run = false;
-  bool task5_run = false;
-  ThreadTaskRunnerHandle::Get()->PostDelayedTask(
-      FROM_HERE, BindLambdaForTesting([&]() { task1_run = true; }),
-      TimeDelta::FromMilliseconds(10));
-
-  ThreadTaskRunnerHandle::Get()->PostDelayedTask(
-      FROM_HERE, BindLambdaForTesting([&]() {
-        task2_run = true;
-        EXPECT_FALSE(task3_run);
-        RunLoop nested_run_loop(RunLoop::Type::kNestableTasksAllowed);
-        nested_run_loop.RunWithTimeout(TimeDelta::FromMilliseconds(20));
-        EXPECT_TRUE(task3_run);
-        EXPECT_TRUE(task4_run);
-      }),
-      TimeDelta::FromMilliseconds(20));
-
-  ThreadTaskRunnerHandle::Get()->PostDelayedTask(
-      FROM_HERE, BindLambdaForTesting([&]() { task3_run = true; }),
-      TimeDelta::FromMilliseconds(30));
-
-  ThreadTaskRunnerHandle::Get()->PostDelayedTask(
-      FROM_HERE, BindLambdaForTesting([&]() { task4_run = true; }),
-      TimeDelta::FromMilliseconds(40));
-
-  ThreadTaskRunnerHandle::Get()->PostDelayedTask(
-      FROM_HERE, BindLambdaForTesting([&]() { task5_run = true; }),
-      TimeDelta::FromSeconds(10));
-
-  run_loop_.RunWithTimeout(TimeDelta::FromMilliseconds(40));
-  EXPECT_TRUE(task1_run);
-  EXPECT_TRUE(task2_run);
-  EXPECT_TRUE(task3_run);
-  EXPECT_TRUE(task4_run);
-  EXPECT_FALSE(task5_run);
-}
-
-TEST_P(RunLoopTest, NestedRunWithTimeoutWhereInnerLoopHasALongerTimeout) {
-  // SimpleSingleThreadTaskRunner doesn't support delayed tasks.
-  if (GetParam() == RunLoopTestType::kTestDelegate)
-    return;
-
-  bool task1_run = false;
-  bool task2_run = false;
-  bool task3_run = false;
-  bool task4_run = false;
-  bool task5_run = false;
-  ThreadTaskRunnerHandle::Get()->PostDelayedTask(
-      FROM_HERE, BindLambdaForTesting([&]() { task1_run = true; }),
-      TimeDelta::FromMilliseconds(10));
-
-  ThreadTaskRunnerHandle::Get()->PostDelayedTask(
-      FROM_HERE, BindLambdaForTesting([&]() {
-        task2_run = true;
-        EXPECT_FALSE(task3_run);
-        RunLoop nested_run_loop(RunLoop::Type::kNestableTasksAllowed);
-        nested_run_loop.RunWithTimeout(TimeDelta::FromMilliseconds(50));
-        EXPECT_TRUE(task3_run);
-        EXPECT_TRUE(task4_run);
-      }),
-      TimeDelta::FromMilliseconds(20));
-
-  ThreadTaskRunnerHandle::Get()->PostDelayedTask(
-      FROM_HERE, BindLambdaForTesting([&]() { task3_run = true; }),
-      TimeDelta::FromMilliseconds(30));
-
-  ThreadTaskRunnerHandle::Get()->PostDelayedTask(
-      FROM_HERE, BindLambdaForTesting([&]() { task4_run = true; }),
-      TimeDelta::FromMilliseconds(40));
-
-  ThreadTaskRunnerHandle::Get()->PostDelayedTask(
-      FROM_HERE, BindLambdaForTesting([&]() { task5_run = true; }),
-      TimeDelta::FromMilliseconds(50));
-
-  run_loop_.RunWithTimeout(TimeDelta::FromMilliseconds(40));
-  EXPECT_TRUE(task1_run);
-  EXPECT_TRUE(task2_run);
-  EXPECT_TRUE(task3_run);
-  EXPECT_TRUE(task4_run);
-  EXPECT_TRUE(task5_run);
-}
-
 // Verify that the QuitWhenIdleClosure() can run after the RunLoop has been
 // deleted. It should have no effect.
 TEST_P(RunLoopTest, QuitWhenIdleClosureAfterRunLoopScope) {
@@ -420,8 +314,8 @@ TEST_P(RunLoopTest, QuitFromOtherSequence) {
       other_thread.task_runner();
 
   // Always expected to run before asynchronous Quit() kicks in.
-  ThreadTaskRunnerHandle::Get()->PostTask(FROM_HERE,
-                                          MakeExpectedRunClosure(FROM_HERE));
+  SingleThreadTaskRunner::GetCurrentDefault()->PostTask(
+      FROM_HERE, MakeExpectedRunClosure(FROM_HERE));
 
   WaitableEvent loop_was_quit(WaitableEvent::ResetPolicy::MANUAL,
                               WaitableEvent::InitialState::NOT_SIGNALED);
@@ -435,8 +329,8 @@ TEST_P(RunLoopTest, QuitFromOtherSequence) {
   // Anything that's posted after the Quit closure was posted back to this
   // sequence shouldn't get a chance to run.
   loop_was_quit.Wait();
-  ThreadTaskRunnerHandle::Get()->PostTask(FROM_HERE,
-                                          MakeExpectedNotRunClosure(FROM_HERE));
+  SingleThreadTaskRunner::GetCurrentDefault()->PostTask(
+      FROM_HERE, MakeExpectedNotRunClosure(FROM_HERE));
 
   run_loop_.Run();
 }
@@ -449,8 +343,8 @@ TEST_P(RunLoopTest, QuitFromOtherSequenceWithClosure) {
       other_thread.task_runner();
 
   // Always expected to run before asynchronous Quit() kicks in.
-  ThreadTaskRunnerHandle::Get()->PostTask(FROM_HERE,
-                                          MakeExpectedRunClosure(FROM_HERE));
+  SingleThreadTaskRunner::GetCurrentDefault()->PostTask(
+      FROM_HERE, MakeExpectedRunClosure(FROM_HERE));
 
   WaitableEvent loop_was_quit(WaitableEvent::ResetPolicy::MANUAL,
                               WaitableEvent::InitialState::NOT_SIGNALED);
@@ -462,8 +356,8 @@ TEST_P(RunLoopTest, QuitFromOtherSequenceWithClosure) {
   // Anything that's posted after the Quit closure was posted back to this
   // sequence shouldn't get a chance to run.
   loop_was_quit.Wait();
-  ThreadTaskRunnerHandle::Get()->PostTask(FROM_HERE,
-                                          MakeExpectedNotRunClosure(FROM_HERE));
+  SingleThreadTaskRunner::GetCurrentDefault()->PostTask(
+      FROM_HERE, MakeExpectedNotRunClosure(FROM_HERE));
 
   run_loop_.Run();
 }
@@ -477,8 +371,8 @@ TEST_P(RunLoopTest, QuitFromOtherSequenceRacy) {
       other_thread.task_runner();
 
   // Always expected to run before asynchronous Quit() kicks in.
-  ThreadTaskRunnerHandle::Get()->PostTask(FROM_HERE,
-                                          MakeExpectedRunClosure(FROM_HERE));
+  SingleThreadTaskRunner::GetCurrentDefault()->PostTask(
+      FROM_HERE, MakeExpectedRunClosure(FROM_HERE));
 
   other_sequence->PostTask(FROM_HERE, run_loop_.QuitClosure());
 
@@ -494,8 +388,8 @@ TEST_P(RunLoopTest, QuitFromOtherSequenceRacyWithClosure) {
       other_thread.task_runner();
 
   // Always expected to run before asynchronous Quit() kicks in.
-  ThreadTaskRunnerHandle::Get()->PostTask(FROM_HERE,
-                                          MakeExpectedRunClosure(FROM_HERE));
+  SingleThreadTaskRunner::GetCurrentDefault()->PostTask(
+      FROM_HERE, MakeExpectedRunClosure(FROM_HERE));
 
   other_sequence->PostTask(FROM_HERE, run_loop_.QuitClosure());
 
@@ -509,16 +403,16 @@ TEST_P(RunLoopTest, QuitWhenIdleFromOtherSequence) {
   scoped_refptr<SequencedTaskRunner> other_sequence =
       other_thread.task_runner();
 
-  ThreadTaskRunnerHandle::Get()->PostTask(FROM_HERE,
-                                          MakeExpectedRunClosure(FROM_HERE));
+  SingleThreadTaskRunner::GetCurrentDefault()->PostTask(
+      FROM_HERE, MakeExpectedRunClosure(FROM_HERE));
 
   other_sequence->PostTask(
       FROM_HERE,
       base::BindOnce([](RunLoop* run_loop) { run_loop->QuitWhenIdle(); },
                      Unretained(&run_loop_)));
 
-  ThreadTaskRunnerHandle::Get()->PostTask(FROM_HERE,
-                                          MakeExpectedRunClosure(FROM_HERE));
+  SingleThreadTaskRunner::GetCurrentDefault()->PostTask(
+      FROM_HERE, MakeExpectedRunClosure(FROM_HERE));
 
   run_loop_.Run();
 
@@ -533,13 +427,13 @@ TEST_P(RunLoopTest, QuitWhenIdleFromOtherSequenceWithClosure) {
   scoped_refptr<SequencedTaskRunner> other_sequence =
       other_thread.task_runner();
 
-  ThreadTaskRunnerHandle::Get()->PostTask(FROM_HERE,
-                                          MakeExpectedRunClosure(FROM_HERE));
+  SingleThreadTaskRunner::GetCurrentDefault()->PostTask(
+      FROM_HERE, MakeExpectedRunClosure(FROM_HERE));
 
   other_sequence->PostTask(FROM_HERE, run_loop_.QuitWhenIdleClosure());
 
-  ThreadTaskRunnerHandle::Get()->PostTask(FROM_HERE,
-                                          MakeExpectedRunClosure(FROM_HERE));
+  SingleThreadTaskRunner::GetCurrentDefault()->PostTask(
+      FROM_HERE, MakeExpectedRunClosure(FROM_HERE));
 
   run_loop_.Run();
 
@@ -549,59 +443,101 @@ TEST_P(RunLoopTest, QuitWhenIdleFromOtherSequenceWithClosure) {
 
 TEST_P(RunLoopTest, IsRunningOnCurrentThread) {
   EXPECT_FALSE(RunLoop::IsRunningOnCurrentThread());
-  ThreadTaskRunnerHandle::Get()->PostTask(
+  SingleThreadTaskRunner::GetCurrentDefault()->PostTask(
       FROM_HERE,
-      BindOnce([]() { EXPECT_TRUE(RunLoop::IsRunningOnCurrentThread()); }));
-  ThreadTaskRunnerHandle::Get()->PostTask(FROM_HERE, run_loop_.QuitClosure());
+      BindOnce([] { EXPECT_TRUE(RunLoop::IsRunningOnCurrentThread()); }));
+  SingleThreadTaskRunner::GetCurrentDefault()->PostTask(
+      FROM_HERE, run_loop_.QuitClosure());
   run_loop_.Run();
 }
 
 TEST_P(RunLoopTest, IsNestedOnCurrentThread) {
   EXPECT_FALSE(RunLoop::IsNestedOnCurrentThread());
 
-  ThreadTaskRunnerHandle::Get()->PostTask(
-      FROM_HERE, BindOnce([]() {
+  SingleThreadTaskRunner::GetCurrentDefault()->PostTask(
+      FROM_HERE, BindOnce([] {
         EXPECT_FALSE(RunLoop::IsNestedOnCurrentThread());
 
         RunLoop nested_run_loop(RunLoop::Type::kNestableTasksAllowed);
 
-        ThreadTaskRunnerHandle::Get()->PostTask(
-            FROM_HERE, BindOnce([]() {
-              EXPECT_TRUE(RunLoop::IsNestedOnCurrentThread());
-            }));
-        ThreadTaskRunnerHandle::Get()->PostTask(FROM_HERE,
-                                                nested_run_loop.QuitClosure());
+        SingleThreadTaskRunner::GetCurrentDefault()->PostTask(
+            FROM_HERE,
+            BindOnce([] { EXPECT_TRUE(RunLoop::IsNestedOnCurrentThread()); }));
+        SingleThreadTaskRunner::GetCurrentDefault()->PostTask(
+            FROM_HERE, nested_run_loop.QuitClosure());
 
         EXPECT_FALSE(RunLoop::IsNestedOnCurrentThread());
         nested_run_loop.Run();
         EXPECT_FALSE(RunLoop::IsNestedOnCurrentThread());
       }));
 
-  ThreadTaskRunnerHandle::Get()->PostTask(FROM_HERE, run_loop_.QuitClosure());
+  SingleThreadTaskRunner::GetCurrentDefault()->PostTask(
+      FROM_HERE, run_loop_.QuitClosure());
   run_loop_.Run();
 }
 
+TEST_P(RunLoopTest, CannotRunMoreThanOnce) {
+  SingleThreadTaskRunner::GetCurrentDefault()->PostTask(
+      FROM_HERE, run_loop_.QuitClosure());
+  run_loop_.Run();
+  EXPECT_DCHECK_DEATH({ run_loop_.Run(); });
+}
+
+TEST_P(RunLoopTest, CanRunUntilIdleMoreThanOnce) {
+  SingleThreadTaskRunner::GetCurrentDefault()->PostTask(FROM_HERE, DoNothing());
+  run_loop_.RunUntilIdle();
+
+  SingleThreadTaskRunner::GetCurrentDefault()->PostTask(FROM_HERE, DoNothing());
+  run_loop_.RunUntilIdle();
+  run_loop_.RunUntilIdle();
+}
+
+TEST_P(RunLoopTest, CanRunUntilIdleThenRunIfNotQuit) {
+  SingleThreadTaskRunner::GetCurrentDefault()->PostTask(FROM_HERE, DoNothing());
+  run_loop_.RunUntilIdle();
+
+  SingleThreadTaskRunner::GetCurrentDefault()->PostTask(
+      FROM_HERE, run_loop_.QuitClosure());
+  run_loop_.Run();
+}
+
+TEST_P(RunLoopTest, CannotRunUntilIdleThenRunIfQuit) {
+  SingleThreadTaskRunner::GetCurrentDefault()->PostTask(
+      FROM_HERE, run_loop_.QuitClosure());
+  run_loop_.RunUntilIdle();
+
+  SingleThreadTaskRunner::GetCurrentDefault()->PostTask(FROM_HERE, DoNothing());
+  EXPECT_DCHECK_DEATH({ run_loop_.Run(); });
+}
+
+TEST_P(RunLoopTest, CannotRunAgainIfQuitWhenIdle) {
+  SingleThreadTaskRunner::GetCurrentDefault()->PostTask(
+      FROM_HERE, run_loop_.QuitWhenIdleClosure());
+  run_loop_.RunUntilIdle();
+
+  EXPECT_DCHECK_DEATH({ run_loop_.RunUntilIdle(); });
+}
+
 namespace {
 
 class MockNestingObserver : public RunLoop::NestingObserver {
  public:
   MockNestingObserver() = default;
+  MockNestingObserver(const MockNestingObserver&) = delete;
+  MockNestingObserver& operator=(const MockNestingObserver&) = delete;
 
   // RunLoop::NestingObserver:
   MOCK_METHOD0(OnBeginNestedRunLoop, void());
   MOCK_METHOD0(OnExitNestedRunLoop, void());
-
- private:
-  DISALLOW_COPY_AND_ASSIGN(MockNestingObserver);
 };
 
 class MockTask {
  public:
   MockTask() = default;
-  MOCK_METHOD0(Task, void());
+  MockTask(const MockTask&) = delete;
+  MockTask& operator=(const MockTask&) = delete;
 
- private:
-  DISALLOW_COPY_AND_ASSIGN(MockTask);
+  MOCK_METHOD0(Task, void());
 };
 
 }  // namespace
@@ -613,10 +549,10 @@ TEST_P(RunLoopTest, NestingObservers) {
 
   RunLoop::AddNestingObserverOnCurrentThread(&nesting_observer);
 
-  const RepeatingClosure run_nested_loop = BindRepeating([]() {
+  const RepeatingClosure run_nested_loop = BindRepeating([] {
     RunLoop nested_run_loop(RunLoop::Type::kNestableTasksAllowed);
-    ThreadTaskRunnerHandle::Get()->PostTask(FROM_HERE,
-                                            nested_run_loop.QuitClosure());
+    SingleThreadTaskRunner::GetCurrentDefault()->PostTask(
+        FROM_HERE, nested_run_loop.QuitClosure());
     nested_run_loop.Run();
   });
 
@@ -625,12 +561,14 @@ TEST_P(RunLoopTest, NestingObservers) {
   // when exiting each nesting depth. Each one of these tasks is ahead of the
   // QuitClosures as those are only posted at the end of the queue when
   // |run_nested_loop| is executed.
-  ThreadTaskRunnerHandle::Get()->PostTask(FROM_HERE, run_nested_loop);
-  ThreadTaskRunnerHandle::Get()->PostTask(
+  SingleThreadTaskRunner::GetCurrentDefault()->PostTask(FROM_HERE,
+                                                        run_nested_loop);
+  SingleThreadTaskRunner::GetCurrentDefault()->PostTask(
       FROM_HERE,
       base::BindOnce(&MockTask::Task, base::Unretained(&mock_task_a)));
-  ThreadTaskRunnerHandle::Get()->PostTask(FROM_HERE, run_nested_loop);
-  ThreadTaskRunnerHandle::Get()->PostTask(
+  SingleThreadTaskRunner::GetCurrentDefault()->PostTask(FROM_HERE,
+                                                        run_nested_loop);
+  SingleThreadTaskRunner::GetCurrentDefault()->PostTask(
       FROM_HERE,
       base::BindOnce(&MockTask::Task, base::Unretained(&mock_task_b)));
 
@@ -647,13 +585,13 @@ TEST_P(RunLoopTest, NestingObservers) {
   RunLoop::RemoveNestingObserverOnCurrentThread(&nesting_observer);
 }
 
-TEST_P(RunLoopTest, DisallowRunningForTesting) {
-  RunLoop::ScopedDisallowRunningForTesting disallow_running;
+TEST_P(RunLoopTest, DisallowRunning) {
+  ScopedDisallowRunningRunLoop disallow_running;
   EXPECT_DCHECK_DEATH({ run_loop_.RunUntilIdle(); });
 }
 
-TEST_P(RunLoopTest, ExpiredDisallowRunningForTesting) {
-  { RunLoop::ScopedDisallowRunningForTesting disallow_running; }
+TEST_P(RunLoopTest, ExpiredDisallowRunning) {
+  { ScopedDisallowRunningRunLoop disallow_running; }
   // Running should be fine after |disallow_running| goes out of scope.
   run_loop_.RunUntilIdle();
 }
@@ -665,52 +603,10 @@ INSTANTIATE_TEST_SUITE_P(Mock,
                          RunLoopTest,
                          testing::Values(RunLoopTestType::kTestDelegate));
 
-TEST(ScopedRunTimeoutForTestTest, TimesOut) {
-  test::ScopedTaskEnvironment task_environment;
-  RunLoop run_loop;
-
-  static constexpr auto kArbitraryTimeout =
-      base::TimeDelta::FromMilliseconds(10);
-  RunLoop::ScopedRunTimeoutForTest run_timeout(
-      kArbitraryTimeout, MakeExpectedRunAtLeastOnceClosure(FROM_HERE));
-
-  // Since the delayed task will be posted only after the message pump starts
-  // running, the ScopedRunTimeoutForTest will already have started to elapse,
-  // so if Run() exits at the correct time then our delayed task will not run.
-  SequencedTaskRunnerHandle::Get()->PostTask(
-      FROM_HERE,
-      base::BindOnce(base::IgnoreResult(&SequencedTaskRunner::PostDelayedTask),
-                     SequencedTaskRunnerHandle::Get(), FROM_HERE,
-                     MakeExpectedNotRunClosure(FROM_HERE), kArbitraryTimeout));
-
-  // This task should get to run before Run() times-out.
-  SequencedTaskRunnerHandle::Get()->PostDelayedTask(
-      FROM_HERE, MakeExpectedRunClosure(FROM_HERE), kArbitraryTimeout);
-
-  run_loop.Run();
-}
-
-TEST(ScopedRunTimeoutForTestTest, RunTasksUntilTimeout) {
-  test::ScopedTaskEnvironment task_environment;
-  RunLoop run_loop;
-
-  static constexpr auto kArbitraryTimeout =
-      base::TimeDelta::FromMilliseconds(10);
-  RunLoop::ScopedRunTimeoutForTest run_timeout(
-      kArbitraryTimeout, MakeExpectedRunAtLeastOnceClosure(FROM_HERE));
-
-  // Posting a task with the same delay as our timeout, immediately before
-  // calling Run(), means it should get to run. Since this uses QuitWhenIdle(),
-  // the Run() timeout callback should also get to run.
-  SequencedTaskRunnerHandle::Get()->PostDelayedTask(
-      FROM_HERE, MakeExpectedRunClosure(FROM_HERE), kArbitraryTimeout);
-
-  run_loop.Run();
-}
-
 TEST(RunLoopDeathTest, MustRegisterBeforeInstantiating) {
   TestBoundDelegate unbound_test_delegate_;
-  // RunLoop::RunLoop() should CHECK fetching the ThreadTaskRunnerHandle.
+  // RunLoop::RunLoop() should CHECK fetching the
+  // SingleThreadTaskRunner::CurrentDefaultHandle.
   EXPECT_DEATH_IF_SUPPORTED({ RunLoop(); }, "");
 }
 
@@ -730,13 +626,13 @@ TEST(RunLoopDelegateTest, NestableTasksDontRunInDefaultNestedLoops) {
   // The first task on the main loop will result in a nested run loop. Since
   // it's not kNestableTasksAllowed, no further task should be processed until
   // it's quit.
-  ThreadTaskRunnerHandle::Get()->PostTask(
+  SingleThreadTaskRunner::GetCurrentDefault()->PostTask(
       FROM_HERE,
       BindOnce([](RunLoop* nested_run_loop) { nested_run_loop->Run(); },
                Unretained(&nested_run_loop)));
 
   // Post a task that will fail if it runs inside the nested run loop.
-  ThreadTaskRunnerHandle::Get()->PostTask(
+  SingleThreadTaskRunner::GetCurrentDefault()->PostTask(
       FROM_HERE,
       BindOnce(
           [](const bool& nested_run_loop_ended,
