@@ -1,25 +1,24 @@
-// Copyright 2014 The Chromium Authors. All rights reserved.
+// Copyright 2014 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "chrome/browser/devtools/device/port_forwarding_controller.h"
 
 #include <map>
+#include <memory>
 #include <utility>
 
-#include "base/bind.h"
 #include "base/compiler_specific.h"
+#include "base/functional/bind.h"
 #include "base/json/json_reader.h"
 #include "base/json/json_writer.h"
-#include "base/macros.h"
+#include "base/memory/raw_ptr.h"
 #include "base/memory/singleton.h"
-#include "base/single_thread_task_runner.h"
 #include "base/strings/string_number_conversions.h"
 #include "base/strings/string_split.h"
 #include "base/strings/string_util.h"
-#include "base/task/post_task.h"
+#include "base/task/single_thread_task_runner.h"
 #include "base/threading/thread_checker.h"
-#include "base/threading/thread_task_runner_handle.h"
 #include "chrome/browser/profiles/profile.h"
 #include "chrome/common/pref_names.h"
 #include "components/keyed_service/content/browser_context_dependency_manager.h"
@@ -28,10 +27,13 @@
 #include "content/public/browser/browser_task_traits.h"
 #include "content/public/browser/browser_thread.h"
 #include "content/public/browser/storage_partition.h"
-#include "mojo/public/cpp/bindings/binding.h"
+#include "mojo/public/cpp/bindings/receiver.h"
 #include "net/base/address_list.h"
 #include "net/base/io_buffer.h"
 #include "net/base/net_errors.h"
+#include "net/base/network_isolation_key.h"
+#include "net/dns/public/host_resolver_results.h"
+#include "net/dns/public/resolve_error_info.h"
 #include "net/log/net_log_source.h"
 #include "net/log/net_log_with_source.h"
 #include "net/socket/tcp_client_socket.h"
@@ -66,35 +68,38 @@ const char kPortParam[] = "port";
 const char kConnectionIdParam[] = "connectionId";
 
 static bool ParseNotification(const std::string& json,
-                              std::string* method,
-                              base::Optional<base::Value>* params) {
-  base::Optional<base::Value> value = base::JSONReader::Read(json);
+                              std::string& method,
+                              std::optional<base::Value::Dict>& params) {
+  std::optional<base::Value> value = base::JSONReader::Read(json);
   if (!value || !value->is_dict())
     return false;
 
-  const std::string* method_value = value->FindStringKey(kMethodParam);
+  base::Value::Dict& dict = value->GetDict();
+  std::string* method_value = dict.FindString(kMethodParam);
   if (!method_value)
     return false;
-  *method = *method_value;
+  method = std::move(*method_value);
 
-  auto extracted_param = value->ExtractKey(kParamsParam);
-  if (extracted_param && extracted_param->is_dict())
-    *params = std::move(extracted_param);
+  base::Value::Dict* param_dict = dict.FindDict(kParamsParam);
+  if (param_dict) {
+    params = std::move(*param_dict);
+  }
   return true;
 }
 
 static bool ParseResponse(const std::string& json,
                           int* command_id,
                           int* error_code) {
-  base::Optional<base::Value> value = base::JSONReader::Read(json);
+  std::optional<base::Value> value = base::JSONReader::Read(json);
   if (!value || !value->is_dict())
     return false;
-  base::Optional<int> command_id_opt = value->FindIntKey(kIdParam);
+  const base::Value::Dict& dict = value->GetDict();
+  std::optional<int> command_id_opt = dict.FindInt(kIdParam);
   if (!command_id_opt)
     return false;
   *command_id = *command_id_opt;
 
-  base::Optional<int> error_value = value->FindIntPath(kErrorCodePath);
+  std::optional<int> error_value = dict.FindIntByDottedPath(kErrorCodePath);
   if (error_value)
     *error_code = *error_value;
 
@@ -104,10 +109,10 @@ static bool ParseResponse(const std::string& json,
 static std::string SerializeCommand(int command_id,
                                     const std::string& method,
                                     base::Value params) {
-  base::Value command(base::Value::Type::DICTIONARY);
-  command.SetIntKey(kIdParam, command_id);
-  command.SetStringKey(kMethodParam, method);
-  command.SetKey(kParamsParam, std::move(params));
+  base::Value::Dict command;
+  command.Set(kIdParam, command_id);
+  command.Set(kMethodParam, method);
+  command.Set(kParamsParam, std::move(params));
 
   std::string json_command;
   base::JSONWriter::Write(command, &json_command);
@@ -153,31 +158,41 @@ class PortForwardingHostResolver : public network::ResolveHostClientBase {
                              const std::string& host,
                              int port,
                              ResolveHostCallback resolve_host_callback)
-      : binding_(this),
-        resolve_host_callback_(std::move(resolve_host_callback)) {
+      : resolve_host_callback_(std::move(resolve_host_callback)) {
     DCHECK_CURRENTLY_ON(content::BrowserThread::UI);
-    DCHECK(!binding_);
+    DCHECK(!receiver_.is_bound());
 
-    network::mojom::ResolveHostClientPtr client_ptr;
-    binding_.Bind(mojo::MakeRequest(&client_ptr));
-    binding_.set_connection_error_handler(
-        base::BindOnce(&PortForwardingHostResolver::OnComplete,
-                       base::Unretained(this), net::ERR_FAILED, base::nullopt));
     net::HostPortPair host_port_pair(host, port);
-    content::BrowserContext::GetDefaultStoragePartition(profile)
-        ->GetNetworkContext()
-        ->ResolveHost(host_port_pair, nullptr, std::move(client_ptr));
+    // Intentionally using a HostPortPair because scheme isn't specified.
+    // Use a transient NetworkAnonymizationKey, as there's no need to share
+    // cached DNS results from this request with anything else.
+    profile->GetDefaultStoragePartition()->GetNetworkContext()->ResolveHost(
+        network::mojom::HostResolverHost::NewHostPortPair(
+            std::move(host_port_pair)),
+        net::NetworkAnonymizationKey::CreateTransient(), nullptr,
+        receiver_.BindNewPipeAndPassRemote());
+    receiver_.set_disconnect_handler(base::BindOnce(
+        &PortForwardingHostResolver::OnComplete, base::Unretained(this),
+        net::ERR_NAME_NOT_RESOLVED, net::ResolveErrorInfo(net::ERR_FAILED),
+        /*resolved_addresses=*/std::nullopt,
+        /*endpoint_results_with_metadata=*/std::nullopt));
   }
 
+  PortForwardingHostResolver(const PortForwardingHostResolver&) = delete;
+  PortForwardingHostResolver& operator=(const PortForwardingHostResolver&) =
+      delete;
+
  private:
   ~PortForwardingHostResolver() override {
     DCHECK_CURRENTLY_ON(content::BrowserThread::UI);
   }
 
   // network::mojom::ResolveHostClient:
-  void OnComplete(
-      int result,
-      const base::Optional<net::AddressList>& resolved_addresses) override {
+  void OnComplete(int result,
+                  const net::ResolveErrorInfo& resolve_error_info,
+                  const std::optional<net::AddressList>& resolved_addresses,
+                  const std::optional<net::HostResolverEndpointResults>&
+                      endpoint_results_with_metadata) override {
     DCHECK_CURRENTLY_ON(content::BrowserThread::UI);
 
     if (result < 0) {
@@ -190,10 +205,8 @@ class PortForwardingHostResolver : public network::ResolveHostClientBase {
     delete this;
   }
 
-  mojo::Binding<network::mojom::ResolveHostClient> binding_;
+  mojo::Receiver<network::mojom::ResolveHostClient> receiver_{this};
   ResolveHostCallback resolve_host_callback_;
-
-  DISALLOW_COPY_AND_ASSIGN(PortForwardingHostResolver);
 };
 
 static void ResolveHost(Profile* profile,
@@ -218,6 +231,9 @@ class SocketTunnel {
       new SocketTunnel(profile, std::move(socket), host, port);
   }
 
+  SocketTunnel(const SocketTunnel&) = delete;
+  SocketTunnel& operator=(const SocketTunnel&) = delete;
+
   ~SocketTunnel() { DCHECK_CALLED_ON_VALID_THREAD(thread_checker_); }
 
  private:
@@ -228,12 +244,12 @@ class SocketTunnel {
       : remote_socket_(std::move(socket)),
         pending_writes_(0),
         pending_destruction_(false),
-        adb_thread_runner_(base::ThreadTaskRunnerHandle::Get()) {
+        adb_thread_runner_(base::SingleThreadTaskRunner::GetCurrentDefault()) {
     ResolveHostCallback resolve_host_callback = base::BindOnce(
         &SocketTunnel::OnResolveHostComplete, base::Unretained(this));
-    base::PostTaskWithTraits(FROM_HERE, {content::BrowserThread::UI},
-                             base::BindOnce(&ResolveHost, profile, host, port,
-                                            std::move(resolve_host_callback)));
+    content::GetUIThreadTaskRunner({})->PostTask(
+        FROM_HERE, base::BindOnce(&ResolveHost, profile, host, port,
+                                  std::move(resolve_host_callback)));
   }
 
   void OnResolveHostComplete(net::AddressList resolved_addresses) {
@@ -252,10 +268,10 @@ class SocketTunnel {
   void OnResolved(net::AddressList resolved_addresses) {
     DCHECK_CALLED_ON_VALID_THREAD(thread_checker_);
 
-    host_socket_.reset(new net::TCPClientSocket(resolved_addresses, nullptr,
-                                                nullptr, net::NetLogSource()));
+    host_socket_ = std::make_unique<net::TCPClientSocket>(
+        resolved_addresses, nullptr, nullptr, nullptr, net::NetLogSource());
     int result = host_socket_->Connect(
-        base::Bind(&SocketTunnel::OnConnected, base::Unretained(this)));
+        base::BindOnce(&SocketTunnel::OnConnected, base::Unretained(this)));
     if (result != net::ERR_IO_PENDING)
       OnConnected(result);
   }
@@ -281,13 +297,11 @@ class SocketTunnel {
   void Pump(net::StreamSocket* from, net::StreamSocket* to) {
     DCHECK_CALLED_ON_VALID_THREAD(thread_checker_);
 
-    scoped_refptr<net::IOBuffer> buffer =
-        base::MakeRefCounted<net::IOBuffer>(kBufferSize);
-    int result = from->Read(
-        buffer.get(),
-        kBufferSize,
-        base::Bind(
-            &SocketTunnel::OnRead, base::Unretained(this), from, to, buffer));
+    auto buffer = base::MakeRefCounted<net::IOBufferWithSize>(kBufferSize);
+    int result =
+        from->Read(buffer.get(), kBufferSize,
+                   base::BindOnce(&SocketTunnel::OnRead, base::Unretained(this),
+                                  from, to, buffer));
     if (result != net::ERR_IO_PENDING)
       OnRead(from, to, std::move(buffer), result);
   }
@@ -308,10 +322,11 @@ class SocketTunnel {
         base::MakeRefCounted<net::DrainableIOBuffer>(std::move(buffer), total);
 
     ++pending_writes_;
-    result = to->Write(drainable.get(), total,
-                       base::Bind(&SocketTunnel::OnWritten,
-                                  base::Unretained(this), drainable, from, to),
-                       kPortForwardingControllerTrafficAnnotation);
+    result =
+        to->Write(drainable.get(), total,
+                  base::BindOnce(&SocketTunnel::OnWritten,
+                                 base::Unretained(this), drainable, from, to),
+                  kPortForwardingControllerTrafficAnnotation);
     if (result != net::ERR_IO_PENDING)
       OnWritten(drainable, from, to, result);
   }
@@ -333,8 +348,8 @@ class SocketTunnel {
       ++pending_writes_;
       result =
           to->Write(drainable.get(), drainable->BytesRemaining(),
-                    base::Bind(&SocketTunnel::OnWritten, base::Unretained(this),
-                               drainable, from, to),
+                    base::BindOnce(&SocketTunnel::OnWritten,
+                                   base::Unretained(this), drainable, from, to),
                     kPortForwardingControllerTrafficAnnotation);
       if (result != net::ERR_IO_PENDING)
         OnWritten(drainable, from, to, result);
@@ -365,8 +380,6 @@ class SocketTunnel {
   scoped_refptr<base::SingleThreadTaskRunner> adb_thread_runner_;
 
   THREAD_CHECKER(thread_checker_);
-
-  DISALLOW_COPY_AND_ASSIGN(SocketTunnel);
 };
 
 }  // namespace
@@ -379,6 +392,10 @@ class PortForwardingController::Connection
              scoped_refptr<AndroidDeviceManager::Device> device,
              scoped_refptr<DevToolsAndroidBridge::RemoteBrowser> browser,
              const ForwardingMap& forwarding_map);
+
+  Connection(const Connection&) = delete;
+  Connection& operator=(const Connection&) = delete;
+
   ~Connection() override;
 
   const PortStatusMap& GetPortStatusMap();
@@ -394,9 +411,9 @@ class PortForwardingController::Connection
       content::BrowserThread::UI>;
   friend class base::DeleteHelper<Connection>;
 
-  typedef std::map<int, std::string> ForwardingMap;
-  typedef base::Callback<void(PortStatus)> CommandCallback;
-  typedef std::map<int, CommandCallback> CommandCallbackMap;
+  using ForwardingMap = std::map<int, std::string>;
+  using CommandCallback = base::OnceCallback<void(PortStatus)>;
+  using CommandCallbackMap = std::map<int, CommandCallback>;
 
   void SerializeChanges(const std::string& method,
                         const ForwardingMap& old_map,
@@ -413,8 +430,8 @@ class PortForwardingController::Connection
   void OnFrameRead(const std::string& message) override;
   void OnSocketClosed() override;
 
-  Profile* profile_;
-  PortForwardingController::Registry* registry_;
+  raw_ptr<Profile> profile_;
+  raw_ptr<PortForwardingController::Registry> registry_;
   scoped_refptr<AndroidDeviceManager::Device> device_;
   scoped_refptr<DevToolsAndroidBridge::RemoteBrowser> browser_;
   std::unique_ptr<AndroidDeviceManager::AndroidWebSocket> web_socket_;
@@ -423,8 +440,6 @@ class PortForwardingController::Connection
   ForwardingMap forwarding_map_;
   CommandCallbackMap pending_responses_;
   PortStatusMap port_status_;
-
-  DISALLOW_COPY_AND_ASSIGN(Connection);
 };
 
 PortForwardingController::Connection::Connection(
@@ -481,18 +496,14 @@ void PortForwardingController::Connection::SerializeChanges(
 void PortForwardingController::Connection::SendCommand(
     const std::string& method, int port) {
   DCHECK_CURRENTLY_ON(BrowserThread::UI);
-  base::Value params(base::Value::Type::DICTIONARY);
+  base::Value::Dict params;
   DCHECK(method == kBindMethod || kUnbindMethod == method);
-  params.SetIntKey(kPortParam, port);
+  params.Set(kPortParam, port);
   int id = ++command_id_;
 
   if (method == kBindMethod) {
-    pending_responses_[id] =
-        base::Bind(&Connection::ProcessBindResponse,
-                   base::Unretained(this), port);
-#if BUILDFLAG(DEBUG_DEVTOOLS)
-    port_status_[port] = kStatusConnecting;
-#endif  // BUILDFLAG(DEBUG_DEVTOOLS)
+    pending_responses_[id] = base::BindOnce(&Connection::ProcessBindResponse,
+                                            base::Unretained(this), port);
   } else {
     auto it = port_status_.find(port);
     if (it != port_status_.end() && it->second == kStatusError) {
@@ -501,15 +512,12 @@ void PortForwardingController::Connection::SendCommand(
       return;
     }
 
-    pending_responses_[id] =
-        base::Bind(&Connection::ProcessUnbindResponse,
-                   base::Unretained(this), port);
-#if BUILDFLAG(DEBUG_DEVTOOLS)
-    port_status_[port] = kStatusDisconnecting;
-#endif  // BUILDFLAG(DEBUG_DEVTOOLS)
+    pending_responses_[id] = base::BindOnce(&Connection::ProcessUnbindResponse,
+                                            base::Unretained(this), port);
   }
 
-  web_socket_->SendFrame(SerializeCommand(id, method, std::move(params)));
+  web_socket_->SendFrame(
+      SerializeCommand(id, method, base::Value(std::move(params))));
 }
 
 bool PortForwardingController::Connection::ProcessResponse(
@@ -523,7 +531,7 @@ bool PortForwardingController::Connection::ProcessResponse(
   if (it == pending_responses_.end())
     return false;
 
-  it->second.Run(error_code ? kStatusError : kStatusOK);
+  std::move(it->second).Run(error_code ? kStatusError : kStatusOK);
   pending_responses_.erase(it);
   return true;
 }
@@ -567,17 +575,18 @@ void PortForwardingController::Connection::OnFrameRead(
     return;
 
   std::string method;
-  base::Optional<base::Value> params;
-  if (!ParseNotification(message, &method, &params))
+  std::optional<base::Value::Dict> params;
+  if (!ParseNotification(message, method, params)) {
     return;
+  }
 
   if (method != kAcceptedEvent || !params)
     return;
 
-  base::Optional<int> port = params->FindIntKey(kPortParam);
+  std::optional<int> port = params->FindInt(kPortParam);
   if (!port)
     return;
-  const std::string* connection_id = params->FindStringKey(kConnectionIdParam);
+  const std::string* connection_id = params->FindString(kConnectionIdParam);
   if (!connection_id)
     return;
 
@@ -594,21 +603,21 @@ void PortForwardingController::Connection::OnFrameRead(
   std::string destination_host = tokens[0];
 
   device_->OpenSocket(*connection_id,
-                      base::Bind(&SocketTunnel::StartTunnel, profile_,
-                                 destination_host, destination_port));
+                      base::BindOnce(&SocketTunnel::StartTunnel, profile_,
+                                     destination_host, destination_port));
 }
 
 PortForwardingController::PortForwardingController(Profile* profile)
     : profile_(profile), pref_service_(profile->GetPrefs()) {
   pref_change_registrar_.Init(pref_service_);
-  base::Closure callback = base::Bind(
+  base::RepeatingClosure callback = base::BindRepeating(
       &PortForwardingController::OnPrefsChange, base::Unretained(this));
   pref_change_registrar_.Add(prefs::kDevToolsPortForwardingEnabled, callback);
   pref_change_registrar_.Add(prefs::kDevToolsPortForwardingConfig, callback);
   OnPrefsChange();
 }
 
-PortForwardingController::~PortForwardingController() {}
+PortForwardingController::~PortForwardingController() = default;
 
 PortForwardingController::ForwardingStatus
 PortForwardingController::DeviceListChanged(
@@ -647,9 +656,9 @@ void PortForwardingController::OnPrefsChange() {
   forwarding_map_.clear();
 
   if (pref_service_->GetBoolean(prefs::kDevToolsPortForwardingEnabled)) {
-    const base::Value* value =
-        pref_service_->GetDictionary(prefs::kDevToolsPortForwardingConfig);
-    for (const auto& dict_element : value->DictItems()) {
+    const base::Value::Dict& value =
+        pref_service_->GetDict(prefs::kDevToolsPortForwardingConfig);
+    for (auto dict_element : value) {
       int port_num;
       if (base::StringToInt(dict_element.first, &port_num) &&
           dict_element.second.is_string()) {
