@@ -2,8 +2,6 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
-#include "chrome/browser/extensions/api/file_system/consent_provider_impl.h"
-
 #include <memory>
 #include <string>
 #include <utility>
@@ -14,6 +12,7 @@
 #include "base/memory/ref_counted.h"
 #include "base/run_loop.h"
 #include "chrome/browser/ash/login/users/fake_chrome_user_manager.h"
+#include "chrome/browser/extensions/api/file_system/consent_provider_impl.h"
 #include "chrome/test/base/testing_browser_process.h"
 #include "components/prefs/testing_pref_service.h"
 #include "components/user_manager/scoped_user_manager.h"
@@ -23,6 +22,7 @@
 #include "extensions/common/extension_builder.h"
 #include "extensions/common/manifest.h"
 #include "testing/gtest/include/gtest/gtest.h"
+#include "ui/base/mojom/dialog_button.mojom.h"
 
 using extensions::file_system_api::ConsentProviderImpl;
 using extensions::mojom::ManifestLocation;
@@ -34,7 +34,7 @@ namespace {
 // accessible fields.
 struct TestDelegateState {
   // Used to assign a fake dialog response.
-  ui::DialogButton dialog_button = ui::DIALOG_BUTTON_NONE;
+  ui::mojom::DialogButton dialog_button = ui::mojom::DialogButton::kNone;
 
   // Used to assign fake result of detection the auto launch kiosk mode.
   bool is_auto_launched = false;
@@ -95,7 +95,7 @@ class TestingConsentProviderDelegate
   }
 
   // Use raw_ptr since |state| is owned by owner.
-  base::raw_ptr<TestDelegateState> state_;
+  raw_ptr<TestDelegateState> state_;
 };
 
 // Rewrites result of a consent request from |result| to |log|.
@@ -108,7 +108,7 @@ void OnConsentReceived(ConsentProviderImpl::Consent* log,
 
 class FileSystemApiConsentProviderTest : public testing::Test {
  public:
-  FileSystemApiConsentProviderTest() {}
+  FileSystemApiConsentProviderTest() = default;
 
   void SetUp() override {
     testing_pref_service_ = std::make_unique<TestingPrefServiceSimple>();
@@ -117,19 +117,20 @@ class FileSystemApiConsentProviderTest : public testing::Test {
     user_manager_ = new ash::FakeChromeUserManager;
     scoped_user_manager_enabler_ =
         std::make_unique<user_manager::ScopedUserManager>(
-            base::WrapUnique(user_manager_));
+            base::WrapUnique(user_manager_.get()));
   }
 
   void TearDown() override {
     scoped_user_manager_enabler_.reset();
     user_manager_ = nullptr;
-    testing_pref_service_.reset();
     TestingBrowserProcess::GetGlobal()->SetLocalState(nullptr);
+    testing_pref_service_.reset();
   }
 
  protected:
   std::unique_ptr<TestingPrefServiceSimple> testing_pref_service_;
-  ash::FakeChromeUserManager* user_manager_;  // Owned by the scope enabler.
+  raw_ptr<ash::FakeChromeUserManager, DanglingUntriaged>
+      user_manager_;  // Owned by the scope enabler.
   std::unique_ptr<user_manager::ScopedUserManager> scoped_user_manager_enabler_;
   content::BrowserTaskEnvironment task_environment_;
 };
@@ -193,10 +194,9 @@ TEST_F(FileSystemApiConsentProviderTest, ForKioskApps) {
             .SetManifestKey("kiosk_enabled", true)
             .SetManifestKey("kiosk_only", true)
             .Build());
-    user_manager_->AddKioskAppUser(
-        AccountId::FromUserEmail(auto_launch_kiosk_app->id()));
-    user_manager_->LoginUser(
+    auto* auto_user = user_manager_->AddKioskAppUser(
         AccountId::FromUserEmail(auto_launch_kiosk_app->id()));
+    user_manager_->LoginUser(auto_user->GetAccountId());
 
     TestDelegateState state;
     state.is_auto_launched = true;
@@ -222,13 +222,12 @@ TEST_F(FileSystemApiConsentProviderTest, ForKioskApps) {
           .SetManifestKey("kiosk_enabled", true)
           .SetManifestKey("kiosk_only", true)
           .Build());
-  user_manager::User* const manual_kiosk_app_user =
-      user_manager_->AddKioskAppUser(
-          AccountId::FromUserEmail(manual_launch_kiosk_app->id()));
-  user_manager_->KioskAppLoggedIn(manual_kiosk_app_user);
+  auto* manual_user = user_manager_->AddKioskAppUser(
+      AccountId::FromUserEmail(manual_launch_kiosk_app->id()));
+  user_manager_->LoginUser(manual_user->GetAccountId());
   {
     TestDelegateState state;
-    state.dialog_button = ui::DIALOG_BUTTON_OK;
+    state.dialog_button = ui::mojom::DialogButton::kOk;
     ConsentProviderImpl provider(
         std::make_unique<TestingConsentProviderDelegate>(&state));
     EXPECT_TRUE(provider.IsGrantable(*manual_launch_kiosk_app));
@@ -249,7 +248,7 @@ TEST_F(FileSystemApiConsentProviderTest, ForKioskApps) {
   // after rejecting by a user.
   {
     TestDelegateState state;
-    state.dialog_button = ui::DIALOG_BUTTON_CANCEL;
+    state.dialog_button = ui::mojom::DialogButton::kCancel;
     ConsentProviderImpl provider(
         std::make_unique<TestingConsentProviderDelegate>(&state));
     EXPECT_TRUE(provider.IsGrantable(*manual_launch_kiosk_app));
