@@ -1,4 +1,4 @@
-// Copyright 2013 The Chromium Authors. All rights reserved.
+// Copyright 2013 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -8,14 +8,17 @@
 
 #include <algorithm>
 #include <memory>
+#include <optional>
 #include <utility>
 #include <vector>
 
 #include "base/i18n/rtl.h"
 #include "base/lazy_instance.h"
+#include "base/memory/raw_ptr.h"
 #include "base/strings/string_number_conversions.h"
 #include "base/strings/utf_string_conversions.h"
 #include "base/time/time.h"
+#include "build/chromeos_buildflags.h"
 #include "chrome/browser/extensions/api/sessions/session_id.h"
 #include "chrome/browser/extensions/api/tabs/windows_util.h"
 #include "chrome/browser/extensions/extension_tab_util.h"
@@ -31,16 +34,19 @@
 #include "chrome/browser/ui/browser_live_tab_context.h"
 #include "chrome/browser/ui/tabs/tab_strip_model.h"
 #include "components/sessions/content/content_live_tab.h"
+#include "components/sessions/core/tab_restore_service.h"
 #include "components/sync_sessions/open_tabs_ui_delegate.h"
 #include "components/sync_sessions/session_sync_service.h"
 #include "components/sync_sessions/synced_session.h"
+#include "components/tab_groups/tab_group_color.h"
 #include "components/url_formatter/url_formatter.h"
 #include "content/public/browser/web_contents.h"
 #include "extensions/browser/extension_function_dispatcher.h"
 #include "extensions/browser/extension_function_registry.h"
 #include "extensions/browser/extension_system.h"
 #include "extensions/common/error_utils.h"
-#include "ui/base/layout.h"
+#include "extensions/common/mojom/context_type.mojom.h"
+#include "ui/base/mojom/window_show_state.mojom.h"
 
 namespace extensions {
 
@@ -65,7 +71,7 @@ const char kRestoreInIncognitoError[] =
 // descending modified_time (i.e., most recent first).
 bool SortSessionsByRecency(const sync_sessions::SyncedSession* s1,
                            const sync_sessions::SyncedSession* s2) {
-  return s1->modified_time > s2->modified_time;
+  return s1->GetModifiedTime() > s2->GetModifiedTime();
 }
 
 // Comparator function for use with std::sort that will sort tabs in a window
@@ -81,115 +87,131 @@ api::tabs::Tab CreateTabModelHelper(
     int index,
     bool pinned,
     bool active,
-    const Extension* extension) {
+    const Extension* extension,
+    mojom::ContextType context) {
   api::tabs::Tab tab_struct;
 
   const GURL& url = current_navigation.virtual_url();
   std::string title = base::UTF16ToUTF8(current_navigation.title());
 
-  tab_struct.session_id.reset(new std::string(session_id));
-  tab_struct.url.reset(new std::string(url.spec()));
-  tab_struct.fav_icon_url.reset(
-      new std::string(current_navigation.favicon_url().spec()));
+  tab_struct.session_id = session_id;
+  tab_struct.url = url.spec();
+  tab_struct.fav_icon_url = current_navigation.favicon_url().spec();
   if (!title.empty()) {
-    tab_struct.title.reset(new std::string(title));
+    tab_struct.title = title;
   } else {
-    tab_struct.title.reset(new std::string(
-        base::UTF16ToUTF8(url_formatter::FormatUrl(url))));
+    tab_struct.title = base::UTF16ToUTF8(url_formatter::FormatUrl(url));
   }
   tab_struct.index = index;
   tab_struct.pinned = pinned;
   tab_struct.active = active;
-  ExtensionTabUtil::ScrubTabForExtension(extension, nullptr, &tab_struct);
+
+  ExtensionTabUtil::ScrubTabBehavior scrub_tab_behavior =
+      ExtensionTabUtil::GetScrubTabBehavior(extension, context, url);
+  ExtensionTabUtil::ScrubTabForExtension(extension, nullptr, &tab_struct,
+                                         scrub_tab_behavior);
   return tab_struct;
 }
 
-std::unique_ptr<api::windows::Window> CreateWindowModelHelper(
-    std::unique_ptr<std::vector<api::tabs::Tab>> tabs,
+api::windows::Window CreateWindowModelHelper(
+    std::vector<api::tabs::Tab> tabs,
     const std::string& session_id,
     const api::windows::WindowType& type,
     const api::windows::WindowState& state) {
-  std::unique_ptr<api::windows::Window> window_struct(new api::windows::Window);
-  window_struct->tabs = std::move(tabs);
-  window_struct->session_id.reset(new std::string(session_id));
-  window_struct->incognito = false;
-  window_struct->always_on_top = false;
-  window_struct->focused = false;
-  window_struct->type = type;
-  window_struct->state = state;
+  api::windows::Window window_struct;
+  window_struct.tabs = std::move(tabs);
+  window_struct.session_id = session_id;
+  window_struct.incognito = false;
+  window_struct.always_on_top = false;
+  window_struct.focused = false;
+  window_struct.type = type;
+  window_struct.state = state;
   return window_struct;
 }
 
-std::unique_ptr<api::sessions::Session> CreateSessionModelHelper(
+api::sessions::Session CreateSessionModelHelper(
     int last_modified,
-    std::unique_ptr<api::tabs::Tab> tab,
-    std::unique_ptr<api::windows::Window> window) {
-  std::unique_ptr<api::sessions::Session> session_struct(
-      new api::sessions::Session());
-  session_struct->last_modified = last_modified;
-  if (tab)
-    session_struct->tab = std::move(tab);
-  else if (window)
-    session_struct->window = std::move(window);
-  else
+    std::optional<api::tabs::Tab> tab,
+    std::optional<api::windows::Window> window,
+    std::optional<api::tab_groups::TabGroup> group) {
+  api::sessions::Session session_struct;
+  session_struct.last_modified = last_modified;
+  if (tab) {
+    session_struct.tab = std::move(*tab);
+  } else if (window) {
+    session_struct.window = std::move(*window);
+  } else if (group) {
+    // TODO(crbug.com/40757179): Implement group support.
+    NOTIMPLEMENTED();
+  } else {
     NOTREACHED();
+  }
   return session_struct;
 }
 
-bool is_window_entry(const sessions::TabRestoreService::Entry& entry) {
-  return entry.type == sessions::TabRestoreService::WINDOW;
+bool is_window_entry(const sessions::tab_restore::Entry& entry) {
+  return entry.type == sessions::tab_restore::Type::WINDOW;
 }
 
 }  // namespace
 
 api::tabs::Tab SessionsGetRecentlyClosedFunction::CreateTabModel(
-    const sessions::TabRestoreService::Tab& tab,
+    const sessions::tab_restore::Tab& tab,
     bool active) {
   return CreateTabModelHelper(tab.navigations[tab.current_navigation_index],
                               base::NumberToString(tab.id.id()),
                               tab.tabstrip_index, tab.pinned, active,
-                              extension());
+                              extension(), source_context_type());
 }
 
-std::unique_ptr<api::windows::Window>
-SessionsGetRecentlyClosedFunction::CreateWindowModel(
-    const sessions::TabRestoreService::Window& window) {
+api::windows::Window SessionsGetRecentlyClosedFunction::CreateWindowModel(
+    const sessions::tab_restore::Window& window) {
   DCHECK(!window.tabs.empty());
 
-  auto tabs = std::make_unique<std::vector<api::tabs::Tab>>();
-  for (const auto& tab : window.tabs)
-    tabs->push_back(
+  std::vector<api::tabs::Tab> tabs;
+  for (const auto& tab : window.tabs) {
+    tabs.push_back(
         CreateTabModel(*tab, tab->tabstrip_index == window.selected_tab_index));
+  }
 
   return CreateWindowModelHelper(
       std::move(tabs), base::NumberToString(window.id.id()),
-      api::windows::WINDOW_TYPE_NORMAL, api::windows::WINDOW_STATE_NORMAL);
+      api::windows::WindowType::kNormal, api::windows::WindowState::kNormal);
 }
 
-std::unique_ptr<api::sessions::Session>
-SessionsGetRecentlyClosedFunction::CreateSessionModel(
-    const sessions::TabRestoreService::Entry& entry) {
-  std::unique_ptr<api::tabs::Tab> tab;
-  std::unique_ptr<api::windows::Window> window;
+api::tab_groups::TabGroup SessionsGetRecentlyClosedFunction::CreateGroupModel(
+    const sessions::tab_restore::Group& group) {
+  DCHECK(!group.tabs.empty());
+
+  return ExtensionTabUtil::CreateTabGroupObject(group.group_id,
+                                                group.visual_data);
+}
+
+api::sessions::Session SessionsGetRecentlyClosedFunction::CreateSessionModel(
+    const sessions::tab_restore::Entry& entry) {
+  std::optional<api::tabs::Tab> tab;
+  std::optional<api::windows::Window> window;
+  std::optional<api::tab_groups::TabGroup> group;
   switch (entry.type) {
-    case sessions::TabRestoreService::TAB:
-      tab.reset(new api::tabs::Tab(CreateTabModel(
-          static_cast<const sessions::TabRestoreService::Tab&>(entry), false)));
+    case sessions::tab_restore::Type::TAB:
+      tab = CreateTabModel(
+          static_cast<const sessions::tab_restore::Tab&>(entry), false);
       break;
-    case sessions::TabRestoreService::WINDOW:
+    case sessions::tab_restore::Type::WINDOW:
       window = CreateWindowModel(
-          static_cast<const sessions::TabRestoreService::Window&>(entry));
+          static_cast<const sessions::tab_restore::Window&>(entry));
       break;
-    default:
-      NOTREACHED();
+    case sessions::tab_restore::Type::GROUP:
+      group = CreateGroupModel(
+          static_cast<const sessions::tab_restore::Group&>(entry));
   }
   return CreateSessionModelHelper(entry.timestamp.ToTimeT(), std::move(tab),
-                                  std::move(window));
+                                  std::move(window), std::move(group));
 }
 
 ExtensionFunction::ResponseAction SessionsGetRecentlyClosedFunction::Run() {
-  std::unique_ptr<GetRecentlyClosed::Params> params(
-      GetRecentlyClosed::Params::Create(*args_));
+  std::optional<GetRecentlyClosed::Params> params =
+      GetRecentlyClosed::Params::Create(args());
   EXTENSION_FUNCTION_VALIDATE(params);
   int max_results = api::sessions::MAX_SESSION_RESULTS;
   if (params->filter && params->filter->max_results)
@@ -202,7 +224,7 @@ ExtensionFunction::ResponseAction SessionsGetRecentlyClosedFunction::Run() {
       TabRestoreServiceFactory::GetForProfile(
           Profile::FromBrowserContext(browser_context()));
 
-  // TabRestoreServiceFactory::GetForProfile() can return NULL (i.e., when in
+  // TabRestoreServiceFactory::GetForProfile() can return nullptr (i.e., when in
   // incognito mode)
   if (!tab_restore_service) {
     DCHECK(browser_context()->IsOffTheRecord())
@@ -213,8 +235,23 @@ ExtensionFunction::ResponseAction SessionsGetRecentlyClosedFunction::Run() {
   // List of entries. They are ordered from most to least recent.
   // We prune the list to contain max 25 entries at any time and removes
   // uninteresting entries.
+  int counter = 0;
   for (const auto& entry : tab_restore_service->entries()) {
-    result.push_back(std::move(*CreateSessionModel(*entry)));
+    // TODO(crbug.com/40757179): Support group entries in the Sessions API,
+    // rather than sharding the group out into individual tabs.
+    if (entry->type == sessions::tab_restore::Type::GROUP) {
+      auto& group = static_cast<const sessions::tab_restore::Group&>(*entry);
+      for (const auto& tab : group.tabs)
+        if (counter++ < max_results)
+          result.push_back(CreateSessionModel(*tab));
+        else
+          break;
+    } else {
+      if (counter++ < max_results)
+        result.push_back(CreateSessionModel(*entry));
+      else
+        break;
+    }
   }
 
   return RespondNow(ArgumentList(GetRecentlyClosed::Results::Create(result)));
@@ -228,20 +265,24 @@ api::tabs::Tab SessionsGetDevicesFunction::CreateTabModel(
   std::string session_id = SessionId(session_tag, tab.tab_id.id()).ToString();
   return CreateTabModelHelper(
       tab.navigations[tab.normalized_navigation_index()], session_id, tab_index,
-      tab.pinned, active, extension());
+      tab.pinned, active, extension(), source_context_type());
 }
 
-std::unique_ptr<api::windows::Window>
+std::optional<api::windows::Window>
 SessionsGetDevicesFunction::CreateWindowModel(
     const sessions::SessionWindow& window,
     const std::string& session_tag) {
   DCHECK(!window.tabs.empty());
 
+  // Ignore app popup window for now because we do not have a corresponding
+  // api::windows::WindowType value.
+  if (window.type == sessions::SessionWindow::TYPE_APP_POPUP)
+    return std::nullopt;
+
   // Prune tabs that are not syncable or are NewTabPage. Then, sort the tabs
   // from most recent to least recent.
   std::vector<const sessions::SessionTab*> tabs_in_window;
-  for (size_t i = 0; i < window.tabs.size(); ++i) {
-    const sessions::SessionTab* tab = window.tabs[i].get();
+  for (const auto& tab : window.tabs) {
     if (tab->navigations.empty())
       continue;
     const sessions::SerializedNavigationEntry& current_navigation =
@@ -251,98 +292,114 @@ SessionsGetDevicesFunction::CreateWindowModel(
             Profile::FromBrowserContext(browser_context()))) {
       continue;
     }
-    tabs_in_window.push_back(tab);
+    tabs_in_window.push_back(tab.get());
   }
   if (tabs_in_window.empty())
-    return nullptr;
+    return std::nullopt;
   std::sort(tabs_in_window.begin(), tabs_in_window.end(), SortTabsByRecency);
 
-  std::unique_ptr<std::vector<api::tabs::Tab>> tabs(
-      new std::vector<api::tabs::Tab>());
+  std::vector<api::tabs::Tab> tabs;
   for (size_t i = 0; i < tabs_in_window.size(); ++i) {
-    tabs->push_back(CreateTabModel(session_tag, *tabs_in_window[i], i,
-                                   window.selected_tab_index == (int)i));
+    bool is_active = false;
+    if (window.selected_tab_index != -1) {
+      SessionID selected_tab_id =
+          window.tabs[window.selected_tab_index]->tab_id;
+      SessionID current_tab_id = tabs_in_window[i]->tab_id;
+      is_active = selected_tab_id == current_tab_id;
+    }
+    tabs.push_back(
+        CreateTabModel(session_tag, *tabs_in_window[i], i, is_active));
   }
 
   std::string session_id =
       SessionId(session_tag, window.window_id.id()).ToString();
 
-  api::windows::WindowType type = api::windows::WINDOW_TYPE_NONE;
+  api::windows::WindowType type = api::windows::WindowType::kNone;
   switch (window.type) {
-    case sessions::SessionWindow::TYPE_TABBED:
-      type = api::windows::WINDOW_TYPE_NORMAL;
+    case sessions::SessionWindow::TYPE_NORMAL:
+      type = api::windows::WindowType::kNormal;
       break;
     case sessions::SessionWindow::TYPE_POPUP:
-      type = api::windows::WINDOW_TYPE_POPUP;
+      type = api::windows::WindowType::kPopup;
       break;
+    case sessions::SessionWindow::TYPE_APP:
+      type = api::windows::WindowType::kApp;
+      break;
+    case sessions::SessionWindow::TYPE_DEVTOOLS:
+      type = api::windows::WindowType::kDevtools;
+      break;
+    case sessions::SessionWindow::TYPE_APP_POPUP:
+#if BUILDFLAG(IS_CHROMEOS)
+    case sessions::SessionWindow::TYPE_CUSTOM_TAB:
+#endif
+      NOTREACHED();
   }
 
-  api::windows::WindowState state = api::windows::WINDOW_STATE_NONE;
+  api::windows::WindowState state = api::windows::WindowState::kNone;
   switch (window.show_state) {
-    case ui::SHOW_STATE_NORMAL:
-      state = api::windows::WINDOW_STATE_NORMAL;
+    case ui::mojom::WindowShowState::kNormal:
+      state = api::windows::WindowState::kNormal;
       break;
-    case ui::SHOW_STATE_MINIMIZED:
-      state = api::windows::WINDOW_STATE_MINIMIZED;
+    case ui::mojom::WindowShowState::kMinimized:
+      state = api::windows::WindowState::kMinimized;
       break;
-    case ui::SHOW_STATE_MAXIMIZED:
-      state = api::windows::WINDOW_STATE_MAXIMIZED;
+    case ui::mojom::WindowShowState::kMaximized:
+      state = api::windows::WindowState::kMaximized;
       break;
-    case ui::SHOW_STATE_FULLSCREEN:
-      state = api::windows::WINDOW_STATE_FULLSCREEN;
+    case ui::mojom::WindowShowState::kFullscreen:
+      state = api::windows::WindowState::kFullscreen;
       break;
-    case ui::SHOW_STATE_DEFAULT:
-    case ui::SHOW_STATE_INACTIVE:
-    case ui::SHOW_STATE_END:
+    case ui::mojom::WindowShowState::kDefault:
+    case ui::mojom::WindowShowState::kInactive:
+    case ui::mojom::WindowShowState::kEnd:
       break;
   }
 
-  std::unique_ptr<api::windows::Window> window_struct(
-      CreateWindowModelHelper(std::move(tabs), session_id, type, state));
+  api::windows::Window window_struct =
+      CreateWindowModelHelper(std::move(tabs), session_id, type, state);
   // TODO(dwankri): Dig deeper to resolve bounds not being optional, so closed
   // windows in GetRecentlyClosed can have set values in Window helper.
-  window_struct->left.reset(new int(window.bounds.x()));
-  window_struct->top.reset(new int(window.bounds.y()));
-  window_struct->width.reset(new int(window.bounds.width()));
-  window_struct->height.reset(new int(window.bounds.height()));
+  window_struct.left = window.bounds.x();
+  window_struct.top = window.bounds.y();
+  window_struct.width = window.bounds.width();
+  window_struct.height = window.bounds.height();
 
   return window_struct;
 }
 
-std::unique_ptr<api::sessions::Session>
+std::optional<api::sessions::Session>
 SessionsGetDevicesFunction::CreateSessionModel(
     const sessions::SessionWindow& window,
     const std::string& session_tag) {
-  std::unique_ptr<api::windows::Window> window_model(
-      CreateWindowModel(window, session_tag));
+  std::optional<api::windows::Window> window_model =
+      CreateWindowModel(window, session_tag);
   // There is a chance that after pruning uninteresting tabs the window will be
   // empty.
-  return !window_model
-             ? nullptr
-             : CreateSessionModelHelper(window.timestamp.ToTimeT(),
-                                        std::unique_ptr<api::tabs::Tab>(),
-                                        std::move(window_model));
+  if (!window_model)
+    return std::nullopt;
+
+  return CreateSessionModelHelper(window.timestamp.ToTimeT(), std::nullopt,
+                                  std::move(window_model), std::nullopt);
 }
 
 api::sessions::Device SessionsGetDevicesFunction::CreateDeviceModel(
     const sync_sessions::SyncedSession* session) {
   int max_results = api::sessions::MAX_SESSION_RESULTS;
   // Already validated in RunAsync().
-  std::unique_ptr<GetDevices::Params> params(
-      GetDevices::Params::Create(*args_));
+  std::optional<GetDevices::Params> params = GetDevices::Params::Create(args());
   if (params->filter && params->filter->max_results)
     max_results = *params->filter->max_results;
 
   api::sessions::Device device_struct;
-  device_struct.info = session->session_name;
-  device_struct.device_name = session->session_name;
+  device_struct.info = session->GetSessionName();
+  device_struct.device_name = session->GetSessionName();
 
   for (auto it = session->windows.begin();
        it != session->windows.end() &&
        static_cast<int>(device_struct.sessions.size()) < max_results;
        ++it) {
-    std::unique_ptr<api::sessions::Session> session_model(
-        CreateSessionModel(it->second->wrapped_window, session->session_tag));
+    std::optional<api::sessions::Session> session_model = CreateSessionModel(
+        it->second->wrapped_window, session->GetSessionTag());
     if (session_model)
       device_struct.sessions.push_back(std::move(*session_model));
   }
@@ -357,15 +414,15 @@ ExtensionFunction::ResponseAction SessionsGetDevicesFunction::Run() {
 
   sync_sessions::OpenTabsUIDelegate* open_tabs =
       service->GetOpenTabsUIDelegate();
-  std::vector<const sync_sessions::SyncedSession*> sessions;
+  std::vector<raw_ptr<const sync_sessions::SyncedSession, VectorExperimental>>
+      sessions;
   // If the user has disabled tab sync, GetOpenTabsUIDelegate() returns null.
   if (!(open_tabs && open_tabs->GetAllForeignSessions(&sessions))) {
     return RespondNow(ArgumentList(
         GetDevices::Results::Create(std::vector<api::sessions::Device>())));
   }
 
-  std::unique_ptr<GetDevices::Params> params(
-      GetDevices::Params::Create(*args_));
+  std::optional<GetDevices::Params> params = GetDevices::Params::Create(args());
   EXTENSION_FUNCTION_VALIDATE(params);
   if (params->filter && params->filter->max_results) {
     EXTENSION_FUNCTION_VALIDATE(*params->filter->max_results >= 0 &&
@@ -375,38 +432,42 @@ ExtensionFunction::ResponseAction SessionsGetDevicesFunction::Run() {
   std::vector<api::sessions::Device> result;
   // Sort sessions from most recent to least recent.
   std::sort(sessions.begin(), sessions.end(), SortSessionsByRecency);
-  for (size_t i = 0; i < sessions.size(); ++i)
-    result.push_back(CreateDeviceModel(sessions[i]));
+  for (const sync_sessions::SyncedSession* session : sessions) {
+    result.push_back(CreateDeviceModel(session));
+  }
 
   return RespondNow(ArgumentList(GetDevices::Results::Create(result)));
 }
 
 ExtensionFunction::ResponseValue SessionsRestoreFunction::GetRestoredTabResult(
     content::WebContents* contents) {
-  std::unique_ptr<api::tabs::Tab> tab(ExtensionTabUtil::CreateTabObject(
-      contents, ExtensionTabUtil::kScrubTab, extension()));
-  std::unique_ptr<api::sessions::Session> restored_session(
-      CreateSessionModelHelper(base::Time::Now().ToTimeT(), std::move(tab),
-                               std::unique_ptr<api::windows::Window>()));
-  return ArgumentList(Restore::Results::Create(*restored_session));
+  ExtensionTabUtil::ScrubTabBehavior scrub_tab_behavior =
+      ExtensionTabUtil::GetScrubTabBehavior(extension(), source_context_type(),
+                                            contents);
+  api::tabs::Tab tab = ExtensionTabUtil::CreateTabObject(
+      contents, scrub_tab_behavior, extension());
+  api::sessions::Session restored_session = CreateSessionModelHelper(
+      base::Time::Now().ToTimeT(), std::move(tab), std::nullopt, std::nullopt);
+  return ArgumentList(Restore::Results::Create(restored_session));
 }
 
 ExtensionFunction::ResponseValue
 SessionsRestoreFunction::GetRestoredWindowResult(int window_id) {
-  Browser* browser = nullptr;
+  WindowController* window_controller = nullptr;
   std::string error;
-  if (!windows_util::GetBrowserFromWindowID(this, window_id, 0, &browser,
-                                            &error)) {
+  if (!windows_util::GetControllerFromWindowID(this, window_id, 0,
+                                               &window_controller, &error)) {
     return Error(error);
   }
-  std::unique_ptr<base::DictionaryValue> window_value(
-      ExtensionTabUtil::CreateWindowValueForExtension(
-          *browser, extension(), ExtensionTabUtil::kPopulateTabs));
-  std::unique_ptr<api::windows::Window> window(
-      api::windows::Window::FromValue(*window_value));
-  return ArgumentList(Restore::Results::Create(*CreateSessionModelHelper(
-      base::Time::Now().ToTimeT(), std::unique_ptr<api::tabs::Tab>(),
-      std::move(window))));
+  base::Value::Dict window_value =
+      window_controller->CreateWindowValueForExtension(
+          extension(), WindowController::kPopulateTabs, source_context_type());
+  std::optional<api::windows::Window> window =
+      api::windows::Window::FromValue(window_value);
+  DCHECK(window);
+  return ArgumentList(Restore::Results::Create(
+      CreateSessionModelHelper(base::Time::Now().ToTimeT(), std::nullopt,
+                               std::move(*window), std::nullopt)));
 }
 
 ExtensionFunction::ResponseValue
@@ -499,7 +560,7 @@ ExtensionFunction::ResponseValue SessionsRestoreFunction::RestoreForeignSession(
   if (!open_tabs)
     return Error(kSessionSyncError);
 
-  const sessions::SessionTab* tab = NULL;
+  const sessions::SessionTab* tab = nullptr;
   if (open_tabs->GetForeignTab(session_id.session_tag(),
                                SessionID::FromSerializedValue(session_id.id()),
                                &tab)) {
@@ -513,9 +574,11 @@ ExtensionFunction::ResponseValue SessionsRestoreFunction::RestoreForeignSession(
   }
 
   // Restoring a full window.
-  std::vector<const sessions::SessionWindow*> windows;
-  if (!open_tabs->GetForeignSession(session_id.session_tag(), &windows))
+  std::vector<const sessions::SessionWindow*> windows =
+      open_tabs->GetForeignSession(session_id.session_tag());
+  if (windows.empty()) {
     return Error(kInvalidSessionIdError, session_id.ToString());
+  }
 
   std::vector<const sessions::SessionWindow*>::const_iterator window =
       windows.begin();
@@ -535,7 +598,7 @@ ExtensionFunction::ResponseValue SessionsRestoreFunction::RestoreForeignSession(
 }
 
 ExtensionFunction::ResponseAction SessionsRestoreFunction::Run() {
-  std::unique_ptr<Restore::Params> params(Restore::Params::Create(*args_));
+  std::optional<Restore::Params> params = Restore::Params::Create(args());
   EXTENSION_FUNCTION_VALIDATE(params);
 
   Profile* profile = Profile::FromBrowserContext(browser_context());
@@ -546,6 +609,9 @@ ExtensionFunction::ResponseAction SessionsRestoreFunction::Run() {
   if (profile != profile->GetOriginalProfile())
     return RespondNow(Error(kRestoreInIncognitoError));
 
+  if (!ExtensionTabUtil::IsTabStripEditable())
+    return RespondNow(Error(ExtensionTabUtil::kTabStripNotEditableError));
+
   if (!params->session_id)
     return RespondNow(RestoreMostRecentlyClosed(browser));
 
@@ -561,7 +627,7 @@ ExtensionFunction::ResponseAction SessionsRestoreFunction::Run() {
 SessionsEventRouter::SessionsEventRouter(Profile* profile)
     : profile_(profile),
       tab_restore_service_(TabRestoreServiceFactory::GetForProfile(profile)) {
-  // TabRestoreServiceFactory::GetForProfile() can return NULL (i.e., when in
+  // TabRestoreServiceFactory::GetForProfile() can return nullptr (i.e., when in
   // incognito mode)
   if (tab_restore_service_) {
     tab_restore_service_->LoadTabsFromLastSession();
@@ -576,15 +642,14 @@ SessionsEventRouter::~SessionsEventRouter() {
 
 void SessionsEventRouter::TabRestoreServiceChanged(
     sessions::TabRestoreService* service) {
-  std::unique_ptr<base::ListValue> args(new base::ListValue());
   EventRouter::Get(profile_)->BroadcastEvent(std::make_unique<Event>(
       events::SESSIONS_ON_CHANGED, api::sessions::OnChanged::kEventName,
-      std::move(args)));
+      base::Value::List()));
 }
 
 void SessionsEventRouter::TabRestoreServiceDestroyed(
     sessions::TabRestoreService* service) {
-  tab_restore_service_ = NULL;
+  tab_restore_service_ = nullptr;
 }
 
 SessionsAPI::SessionsAPI(content::BrowserContext* context)
@@ -593,8 +658,7 @@ SessionsAPI::SessionsAPI(content::BrowserContext* context)
       api::sessions::OnChanged::kEventName);
 }
 
-SessionsAPI::~SessionsAPI() {
-}
+SessionsAPI::~SessionsAPI() = default;
 
 void SessionsAPI::Shutdown() {
   EventRouter::Get(browser_context_)->UnregisterObserver(this);
@@ -609,8 +673,8 @@ SessionsAPI::GetFactoryInstance() {
 }
 
 void SessionsAPI::OnListenerAdded(const EventListenerInfo& details) {
-  sessions_event_router_.reset(
-      new SessionsEventRouter(Profile::FromBrowserContext(browser_context_)));
+  sessions_event_router_ = std::make_unique<SessionsEventRouter>(
+      Profile::FromBrowserContext(browser_context_));
   EventRouter::Get(browser_context_)->UnregisterObserver(this);
 }
 
