@@ -11,12 +11,12 @@ import {EventTracker} from 'chrome://resources/js/event_tracker.js';
 import {changeFolderOpen, deselectItems, selectItem} from './actions.js';
 import {highlightUpdatedItems, trackUpdatedItems} from './api_listener.js';
 import {BookmarkManagerApiProxyImpl} from './bookmark_manager_api_proxy.js';
-import {DropPosition, ROOT_NODE_ID} from './constants.js';
+import {DropPosition} from './constants.js';
 import {Debouncer} from './debouncer.js';
 import type {BookmarksFolderNodeElement} from './folder_node.js';
 import {Store} from './store.js';
 import type {BookmarkElement, BookmarkNode, DragData, DropDestination, NodeMap, ObjectMap, TimerProxy} from './types.js';
-import {canEditNode, canReorderChildren, getDisplayedList, hasChildFolders, isShowingSearch, normalizeNode} from './util.js';
+import {canEditNode, canReorderChildren, getDisplayedList, hasChildFolders, isRootOrChildOfRoot, isShowingSearch, normalizeNode} from './util.js';
 
 interface NormalizedDragData {
   elements: BookmarkNode[];
@@ -620,7 +620,7 @@ export class DndManager {
     }
 
     // We cannot drop between Bookmarks bar and Other bookmarks.
-    if (getBookmarkNode(overElement).parentId === ROOT_NODE_ID) {
+    if (isRootOrChildOfRoot(state, getBookmarkNode(overElement).id)) {
       return DropPosition.NONE;
     }
 
