@@ -1,4 +1,4 @@
-// Copyright 2016 The Chromium Authors. All rights reserved.
+// Copyright 2016 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -8,8 +8,8 @@
 #include <vector>
 
 #include "chrome/browser/ui/webui/downloads/downloads.mojom.h"
-
-#include "mojo/public/cpp/bindings/binding.h"
+#include "mojo/public/cpp/bindings/pending_remote.h"
+#include "mojo/public/cpp/bindings/receiver.h"
 #include "testing/gmock/include/gmock/gmock.h"
 
 class MockPage : public downloads::mojom::Page {
@@ -17,14 +17,14 @@ class MockPage : public downloads::mojom::Page {
   MockPage();
   ~MockPage() override;
 
-  downloads::mojom::PagePtr BindAndGetPtr();
+  mojo::PendingRemote<downloads::mojom::Page> BindAndGetRemote();
 
-  MOCK_METHOD1(RemoveItem, void(int));
-  MOCK_METHOD2(UpdateItem, void(int, downloads::mojom::DataPtr));
-  MOCK_METHOD2(InsertItems, void(int, std::vector<downloads::mojom::DataPtr>));
-  MOCK_METHOD0(ClearAll, void());
+  MOCK_METHOD(void, RemoveItem, (int));
+  MOCK_METHOD(void, UpdateItem, (int, downloads::mojom::DataPtr));
+  MOCK_METHOD(void, InsertItems, (int, std::vector<downloads::mojom::DataPtr>));
+  MOCK_METHOD(void, ClearAll, ());
 
-  mojo::Binding<downloads::mojom::Page> binding_;
+  mojo::Receiver<downloads::mojom::Page> receiver_{this};
 };
 
 #endif  // CHROME_BROWSER_UI_WEBUI_DOWNLOADS_MOCK_DOWNLOADS_PAGE_H_
