@@ -1,4 +1,4 @@
-// Copyright 2014 The Chromium Authors. All rights reserved.
+// Copyright 2014 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,6 +6,15 @@
 #define CHROME_BROWSER_UI_WEBUI_INTERSTITIALS_INTERSTITIAL_UI_H_
 
 #include "content/public/browser/web_ui_controller.h"
+#include "content/public/browser/webui_config.h"
+
+class InterstitialUI;
+
+class InterstitialUIConfig
+    : public content::DefaultWebUIConfig<InterstitialUI> {
+ public:
+  InterstitialUIConfig();
+};
 
 // Handler for chrome://interstitials demonstration pages. This class is not
 // used in displaying any real interstitials.
