@@ -5,7 +5,6 @@
 #ifndef CHROME_BROWSER_UI_WEBUI_SETTINGS_BROWSER_LIFETIME_HANDLER_H_
 #define CHROME_BROWSER_UI_WEBUI_SETTINGS_BROWSER_LIFETIME_HANDLER_H_
 
-#include "build/chromeos_buildflags.h"
 #include "chrome/browser/ui/webui/settings/settings_page_ui_handler.h"
 
 namespace settings {
@@ -27,12 +26,12 @@ class BrowserLifetimeHandler : public SettingsPageUIHandler {
  private:
   void HandleRestart(const base::Value::List& args);
   void HandleRelaunch(const base::Value::List& args);
-#if BUILDFLAG(IS_CHROMEOS_ASH)
+#if BUILDFLAG(IS_CHROMEOS)
   void HandleSignOutAndRestart(const base::Value::List& args);
   void HandleFactoryReset(const base::Value::List& args);
-#endif  // BUILDFLAG(IS_CHROMEOS_ASH)
+#endif  // BUILDFLAG(IS_CHROMEOS)
 
-#if !BUILDFLAG(IS_CHROMEOS_ASH)
+#if !BUILDFLAG(IS_CHROMEOS)
   void HandleGetRelaunchConfirmationDialogDescription(
       const base::Value::List& args);
   void HandleShouldShowRelaunchConfirmationDialog(
