@@ -15,6 +15,7 @@
 #include "components/account_manager_core/chromeos/account_manager_mojo_service.h"
 #include "components/user_manager/user_manager.h"
 #include "google_apis/gaia/gaia_auth_fetcher.h"
+#include "google_apis/gaia/gaia_id.h"
 
 namespace ash {
 
@@ -35,11 +36,20 @@ SigninHelper::ArcHelper::ArcHelper(
 
 SigninHelper::ArcHelper::~ArcHelper() = default;
 
+void SigninHelper::ArcHelper::SetIsAvailableInArc(bool is_available_in_arc) {
+  is_available_in_arc_ = is_available_in_arc;
+}
+
+bool SigninHelper::ArcHelper::IsAvailableInArc() const {
+  return is_available_in_arc_;
+}
+
 void SigninHelper::ArcHelper::OnAccountAdded(
     const account_manager::Account& account) {
   // Don't change ARC availability after reauthentication.
-  if (!is_account_addition_)
+  if (!is_account_addition_) {
     return;
+  }
 
   account_apps_availability_->SetIsAccountAvailableInArc(account,
                                                          is_available_in_arc_);
@@ -50,10 +60,10 @@ SigninHelper::SigninHelper(
     crosapi::AccountManagerMojoService* account_manager_mojo_service,
     const base::RepeatingClosure& close_dialog_closure,
     const base::RepeatingCallback<void(const std::string&, const std::string&)>&
-        show_signin_blocked_error,
+        show_signin_error,
     scoped_refptr<network::SharedURLLoaderFactory> url_loader_factory,
     std::unique_ptr<ArcHelper> arc_helper,
-    const std::string& gaia_id,
+    const GaiaId& gaia_id,
     const std::string& email,
     const std::string& auth_code,
     const std::string& signin_scoped_device_id)
@@ -61,17 +71,16 @@ SigninHelper::SigninHelper(
       account_manager_mojo_service_(account_manager_mojo_service),
       arc_helper_(std::move(arc_helper)),
       close_dialog_closure_(close_dialog_closure),
-      show_signin_blocked_error_(show_signin_blocked_error),
-      account_key_(gaia_id, account_manager::AccountType::kGaia),
+      show_signin_error_(show_signin_error),
+      account_key_(account_manager::AccountKey::FromGaiaId(gaia_id)),
       email_(email),
       url_loader_factory_(std::move(url_loader_factory)),
       gaia_auth_fetcher_(this, gaia::GaiaSource::kChrome, url_loader_factory_) {
   DCHECK(!signin_scoped_device_id.empty());
+  CHECK(show_signin_error_);
+  DCHECK(arc_helper_);
 
-  if (AccountAppsAvailability::IsArcAccountRestrictionsEnabled())
-    DCHECK(arc_helper_);
   if (!IsInitialPrimaryAccount()) {
-    DCHECK(show_signin_blocked_error_);
     restriction_fetcher_ =
         std::make_unique<UserCloudSigninRestrictionPolicyFetcher>(
             email_, url_loader_factory_);
@@ -83,6 +92,10 @@ SigninHelper::SigninHelper(
 
 SigninHelper::~SigninHelper() = default;
 
+bool SigninHelper::IsAvailableInArc() const {
+  return arc_helper_ && arc_helper_->IsAvailableInArc();
+}
+
 void SigninHelper::OnClientOAuthSuccess(const ClientOAuthResult& result) {
   refresh_token_ = result.refresh_token;
   if (!IsInitialPrimaryAccount()) {
@@ -104,13 +117,14 @@ void SigninHelper::OnClientOAuthFailure(const GoogleServiceAuthError& error) {
   LOG(ERROR) << "SigninHelper::OnClientOAuthFailure: couldn't fetch OAuth2 "
                 "token, the error was "
              << error.ToString();
-  // TODO(http://b/279733142): Display an error.
+
+  show_signin_error_.Run(email_, /*hosted_domain=*/std::string());
 
   // Notify `AccountManagerMojoService` about account addition failure and send
   // `error`.
   account_manager_mojo_service_->OnAccountUpsertionFinished(
       account_manager::AccountUpsertionResult::FromError(error));
-  CloseDialogAndExit();
+  Exit();
 }
 
 void SigninHelper::UpsertAccount(const std::string& refresh_token) {
@@ -130,9 +144,7 @@ void SigninHelper::UpsertAccount(const std::string& refresh_token) {
   account_manager_->UpsertAccount(account_key_, email_, refresh_token);
 
   auto new_account = account_manager::Account{account_key_, email_};
-  if (AccountAppsAvailability::IsArcAccountRestrictionsEnabled()) {
-    arc_helper_->OnAccountAdded(new_account);
-  }
+  arc_helper_->OnAccountAdded(new_account);
   // Notify `AccountManagerMojoService` about successful account addition and
   // send the account.
   account_manager_mojo_service_->OnAccountUpsertionFinished(
@@ -154,7 +166,7 @@ void SigninHelper::Exit() {
 // show an error page.
 void SigninHelper::OnGetSecondaryGoogleAccountUsage(
     SigninRestrictionPolicyFetcher::Status status,
-    absl::optional<std::string> policy_result,
+    std::optional<std::string> policy_result,
     const std::string& hosted_domain) {
   base::UmaHistogramEnumeration(kSecondaryGoogleAccountUsageHistogramName,
                                 status);
@@ -187,14 +199,35 @@ void SigninHelper::OnGetSecondaryGoogleAccountUsage(
     return;
   }
 
-  // Enterprise accounts with no restrictions are allow to sign-in.
+  restriction_fetcher_->GetSecondaryAccountAllowedInArcPolicy(
+      /*access_token_fetcher=*/GaiaAccessTokenFetcher::
+          CreateExchangeRefreshTokenForAccessTokenInstance(
+              restriction_fetcher_.get(), url_loader_factory_, refresh_token_),
+      /*callback=*/base::BindOnce(
+          &SigninHelper::OnGetSecondaryAccountAllowedInArcPolicy,
+          weak_factory_.GetWeakPtr()));
+}
+
+void SigninHelper::OnGetSecondaryAccountAllowedInArcPolicy(
+    SigninRestrictionPolicyFetcher::Status status,
+    std::optional<bool> policy_result) {
+  if (status != SigninRestrictionPolicyFetcher::Status::kSuccess) {
+    // If we can't fetch the policy, we don't know if we can sync to ARC.
+    // Block adding the account as a safety measure.
+    ShowSigninBlockedErrorPageAndExit(/*hosted_domain=*/std::string());
+    return;
+  }
+
+  DCHECK(policy_result);
+  arc_helper_->SetIsAvailableInArc(policy_result.value());
+
   UpsertAccount(refresh_token_);
   CloseDialogAndExit();
 }
 
 void SigninHelper::ShowSigninBlockedErrorPageAndExit(
     const std::string& hosted_domain) {
-  show_signin_blocked_error_.Run(email_, hosted_domain);
+  show_signin_error_.Run(email_, hosted_domain);
   RevokeGaiaTokenOnServer();
 }
 
@@ -220,7 +253,7 @@ bool SigninHelper::IsInitialPrimaryAccount() {
   return user_manager::UserManager::Get()
              ->GetPrimaryUser()
              ->GetAccountId()
-             .GetGaiaId() == account_key_.id();
+             .GetGaiaId() == GaiaId(account_key_.id());
 }
 
 account_manager::AccountManager* SigninHelper::GetAccountManager() {
